// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'

var db = null;

angular.module('AprendaApp', ['ionic','ionic.service.core','ionic.service.analytics', 'youtube-embed', 'ngCordova'])
.run(['$cordovaSplashscreen', '$rootScope', '$ionicPlatform', '$cordovaSQLite', '$state', 'sessionUser', '$ionicAnalytics', '$cordovaAppVersion', '$cordovaDevice', function($cordovaSplashscreen, $rootScope, $ionicPlatform, $cordovaSQLite, $state, sessionUser, $ionicAnalytics, $cordovaAppVersion, $cordovaDevice) {
	$ionicPlatform.ready(function() {
		setTimeout(function(){
			$cordovaSplashscreen.hide();
		},500);

		$ionicAnalytics.register();
		
		$cordovaAppVersion.getAppVersion().then(function (version) {
			$ionicAnalytics.setGlobalProperties({
				'versão' : version,
				'plataforma': $cordovaDevice.getDevice().platform,
				'dispositivo': $cordovaDevice.getDevice().model
			});
		});

		// Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
		// for form inputs)
		if(window.cordova && window.cordova.plugins.Keyboard) {
			cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
		}
		if(window.StatusBar) {
			StatusBar.styleDefault();
		}

		if(window.Connection) {
			if(navigator.connection.type == Connection.NONE) {
				$state.go('aprenda.no-connection');
			}
		}

		db = $cordovaSQLite.openDB('aprenda.db');
		$cordovaSQLite.execute(db, "CREATE TABLE IF NOT EXISTS info (sessionID text, sessionUser text)");

		sessionUser.checkUserLogged();
	});
	$rootScope.$on('$stateChangeStart', function(event, toState, $stateParams) {
		if (toState.name == 'aprenda.home') {
			$rootScope.isHome = true;
		} else {
			$rootScope.isHome = false;
		}
		$rootScope.search = false;
	});
}])

.config(['$stateProvider','$urlRouterProvider','$locationProvider','$ionicConfigProvider','$httpProvider', '$ionicAutoTrackProvider',
	function($stateProvider,$urlRouterProvider,$locationProvider,$ionicConfigProvider,$httpProvider, $ionicAutoTrackProvider){
		$ionicConfigProvider.templates.maxPrefetch(0);
		// URL padrão de redirecionamento
		$urlRouterProvider.otherwise('/');

		$ionicAutoTrackProvider.disableTracking('Tap');

		// Configuração das rotas iniciais
		$stateProvider
		// Master page do menu principal
		.state('aprenda', {
			'abstract': true,
			templateUrl: 'app/Shared/aprenda-menu.html'
		})
		// Home do Aprenda
		.state('aprenda.home', {
			url: '/',
			views: {
				'menuContent': {
					templateUrl: 'app/Home/aprenda-home.html',
				}
			}
		})
		// Página para escolher um modo de jogo para uma música (vídeo, tradução)
		.state('game-mode', {
			url: '/jogar/:artUrl/:musUrl/',
			templateUrl: 'app/Game/game-mode.html'
		})
		// Modo vídeo do jogo
		.state('video', {
			url: '/jogar/:artUrl/:musUrl/video/:level/:subID',
			templateUrl: 'app/Game/Video/game-video.html',
			cache: false
		})
		// Modo tradução do jogo
		.state('verse', {
			url: '/jogar/:artUrl/:musUrl/frase/:level/',
			templateUrl: 'app/Game/Verse/game-verse.html'
		})
		// Página para escolher um idioma do modo aleatório (inglês, espanhol)
		.state('random-mode', {
			url: '/jogar-aleatorio/',
			templateUrl: 'app/Game/Random/game-random-mode.html'
		})
		.state('aprenda.rank', {
			url: '/ranking/',
			views: {
				'menuContent': {
					templateUrl: 'app/Rank/rank.html',
				}
			}
		})
		.state('suggest', {
			url: '/procurar/',
			templateUrl: 'app/Suggest/search.html'
		})
		.state('aprenda.rules', {
			url: '/regras/',
			views: {
				'menuContent': {
					templateUrl: 'app/Rules/rules.html',
				}
			}
		})
		.state('login', {
			url: '/login/',
			templateUrl: 'app/Auth/login.html'
		})
		.state('register', {
			url: '/cadastrar/',
			templateUrl: 'app/Auth/register.html'
		})
		.state('aprenda.no-connection', {
			url: '/conexao/',
			views: {
				'menuContent': {
					templateUrl: 'app/Home/no-connection.html',
				}
			}	
		})
		.state('lang', {
			url: '/:lang/',
			templateUrl: 'app/Lang/lang.html'
		})
	}
])

// Factories
.factory('sessionUser', 
	['$http', '$q', '$state', '$rootScope', '$cordovaSQLite', function($http, $q, $state, $rootScope, $cordovaSQLite){
		var loginURL = 'https://auth.vagalume.com.br/'
		  , user = {}
		  , edit = {}
		  , whenLoggedListeners = []
		  , helloIninted = false;

		function connect(service, cb, redirect){
			var accessToken = false
			, userLogin = false;
			if (service == "google") {
				window.plugins.googleplus.login(
					{'iOSApiKey': '363858085852-ak7r4j5a34n8o3t48ciu09stqr7v2418.apps.googleusercontent.com'},
					function (obj) {						
						try {
							accessToken = obj.oauthToken;
						} catch(e) {
							errorHandler(e);
							return;
						}
						initLogin();
					},
					function (msg) {
						cb(normalizeLoginMessage('NO_MSG'));
					}
				);
			} else {
				var resAccessToken = function(token) {					
					try {
						accessToken = token;
					} catch(e) {
						errorHandler(e);
						return;
					}
					initLogin();
				}

				facebookConnectPlugin.getAccessToken(function(result){
					resAccessToken(result)
				},function(){
					facebookConnectPlugin.login(["email"], function(response) {
						if (response.authResponse) {
							facebookConnectPlugin.getAccessToken(function(result){
								resAccessToken(result)
							})
						}
					},function(response){
						cb(normalizeLoginMessage('NO_MSG'));
					});
				})
			}

			function initLogin(){
				var params = '?access_token=' + encodeURI(accessToken)
				+ '&vgl_action=login'
				+ '&vgl_from=' + encodeURI(service)
				+ '&callback=JSON_CALLBACK';
				$http.jsonp(loginURL+'external_login.php'+params).success(function(data){
					if (data && data.sessionID) {
						data.session = {
							ID: data.sessionID,
							User: data.sessionUser
						}
					}
					data = normalizeLoginMessage(data);
					if(data && data.session){
						saveSessionSQLite(data);
						instance.on && instance.on('logged');
					}
					switch(data.code){
						case 'FAIL_LOGIN':
							data.error = 'CANNOT_CONNECT';
							break;
						case 'INVALID_LOGIN':
					}
					cb(data || null);
				});
			}
		}

		function checkUserLogged(){
			var query = 'SELECT sessionID, sessionUser FROM info';
			$cordovaSQLite
				.execute(db, query)
				.then(function(res){
					if(res.rows.length) {
					   $rootScope.userInfo = {
							'sessionID'   : res.rows.item(0).sessionID,
							'sessionUser' : res.rows.item(0).sessionUser,
						}
						$rootScope.$emit('userLogged');
						$rootScope.loggedUser = true;
					} else {
						$rootScope.loggedUser = false;
					}
				})
		}

		function removeSessionSQLite() {
			var query = "DELETE FROM info";
			$cordovaSQLite.execute(db, query);
			checkUserLogged();
		}

		function saveSessionSQLite(data) {
			if (!data || (data && !data.session)) return false;
			removeSessionSQLite();
			var query = "INSERT INTO info (sessionID, sessionUser) VALUES (?,?)";
			$cordovaSQLite.execute(db, query, [data.session.ID, data.session.User]);
			checkUserLogged();
		}

		function fieldErrorMessage(field){
			switch(field){
				case 'name': return 'Nome inválido';
				case 'gender': return 'Gênero inválido';
				case 'email': return 'E-mail inválido';
				case 'emailconf': return 'E-mail inválido';
				case 'pass': return 'Senha inválida';
				case 'passconf': return 'Senha inválido';
				case 'agree': return 'Necessário concordar com termos de uso e privacidade';
				case 'name_show':
				case 'nameshow': return 'Nome inválido';
				case 'login': return 'Login inválido';
				case 'aboutMe': return 'Texto inválido';
				case 'timeout': return 'Erro ao registrar';
			}
			return '';
		}

		function getFieldDataError(field, data){
			var val = data[field];
			switch(field){
				case 'name':
					if(!val) return 'Nome requerido';
				case 'nameshow':
				case 'name_show':
					break;
				case 'login':
					if(!val) return 'Login requerido';
					break;
				case 'emailconf':
					if(val !== data.email) return 'E-mail não confere';
				case 'email':
					if(!val) return 'E-mail requerido';
					break;
				case 'passconf':
				case 'pass':
					if(!val) return 'Senha requerida';
					break;
				case 'agree':
					if(!val) return 'Necessário concordar com os termos de uso';
					break;
				case 'telefone1':
				case 'telefone2':
				case 'telefone3':
					break;
				case 'address':
				case 'bairro':
				case 'cep1':
				case 'cep2':
				case 'cep':
				case 'city':
				case 'state':
				case 'gender':
				case 'countryID':
				case 'noFindMe':
			}
			return null;
		}

		function getDataErrors(data){
			var hasError = false
			  , errors   = {};
			for(var field in data){
				var error = getFieldDataError(field, data);
				if(error) {
					errors[field] = error;
					hasError = true;
				}
			}
			return hasError ? errors : null;
		}

		function getSessionID(){
			return ($rootScope && $rootScope.userInfo && $rootScope.userInfo.sessionID) || false;
		}

		function getSessionUserID(){
			return ($rootScope && $rootScope.userInfo && $rootScope.userInfo.sessionUser) || false;
		}

		function getSessionUserData(callback){
			if(getSessionUserID()){
				requestAction('getSessionUser', null, callback);
			} else {
				setTimeout(function(){
					callback(false);
				}, 1);
			}
		}

		function load(callback){
			getSessionUserData(function(userData){
				if(userData){
					for(var name in userData){
						switch(name){
							case 'nasc':
								edit.nasc = user.nasc = userData.nasc.split('-').reverse().join('/');
								break;
							case 'gender':
								edit[name] = (userData.gender == 'female');
								break;
							case 'countryID':
								user.isBR = userData[name] == 'BR';
								edit[name] = user[name] = userData[name];
								break;
							case 'state':
								edit[name] = user[name] = userData[name];
								break;
							case 'name':
							case 'name_show':
							case 'email':
							case 'address':
							case 'bairro':
							case 'cep':
							case 'city':
							case 'login':
							case 'aboutme':
								edit[name] = user[name] = userData[name];
								break;
							case 'preferences':
								edit.newsletter = user.newsletter = Boolean(userData.preferences.match(/newsletter/));
								break;
							case 'noFindMe':
								edit.noFindMe = user.noFindMe = !userData.noFindMe;
								break;
							case 'publicSocialNetworks':
								edit.publicSocialNetworks = user.publicSocialNetworks = userData.publicSocialNetworks;
								break;
							case 'pass':
						}
					}
					for(var i=1, tel; i<=3; i++){
						tel = userData['ddd' + i];
						tel = tel ? ('('+tel+') ') : '';
						tel = tel + userData['telefone'+i];
						edit['telefone' + i] = user['telefone' + i] = tel;
						edit['gender'] = user['gender'] = userData['gender'];
						edit['user_photo'] = user['user_photo'] = userData['user_photo'];
					}
				}
				callback && callback(user);
			});
			return user;
		}

		function login(user, pass, cb){
			var url = loginURL+'login/json.php?action=logon'
				+ '&login='+user+'&pass='+pass
				+ '&getSession=1'
				+ '&callback=JSON_CALLBACK';
			return $http.jsonp(url)
				.success(function(data){
					data = normalizeLoginMessage(data);
					if(data && data.session){
						saveSessionSQLite(data);
						instance.on && instance.on('logged');
					}
					switch(data.code){
						case 'FAIL_LOGIN':
							data.error = 'CANNOT_CONNECT';
							break;
						case 'INVALID_LOGIN':
					}
					cb(data || null);
				})
				.error(function(){
					cb(false);
				});
		}

		function logoff(callback){
			removeSessionSQLite();
			return requestAction('logoff', null, callback);
		}

		function normalizeLoginMessage(data){
			var success = Boolean(data && (data.success || data.session))
			  , errorCode = (data ? (data.error || data.code) : 'FAIL_LOGIN')
			  , session = (data && data.session) || null
			  , messageMap = {
				'BANNED'            :'Usuário banido',
				'CANNOT_CREATE_USER':'Não foi possível criar uma conta',
				'CANNOT_CONNECT'    :'Não foi possível conectar',
				'DELETED'           :'Usuário excluido',
				'FAIL_LOGIN'        :'Login e/ou senha incorreto',
				'INVALID_LOGIN'     :'Login inválido',
				'INVALID_PASS'      :'Senha inválido',
				'LOGGED'            :'Logado com sucesso',
				'NO_SERVICE'        :'Serviço requerido',
				'NO_USER_DATA'      :'Login e/ou senha incorreto',
				'TRUSTED_LOGGED'    :'Logado com sucesso',
				'VINC_IS_ALREADY'   :'Usuário já existe'
			    };
			switch(errorCode){
				case 'TRUSTED_LOGGED':
				case 'LOGGED':
					success = true;
					break;
				default:
					if(success) {
						for(var i=0, s=whenLoggedListeners.length; i<s; i++){
							whenLoggedListeners[i](user);
						}
						whenLoggedListeners = [];
						$rootScope.$emit('user.logged');
						errorCode = 'LOGGED'
					};
			}
			return {
				success:success,
				code:errorCode,
				session:session || null,
				message:messageMap[errorCode]
			};
		}

		function onFail(field, callback){
			callback(false, {field:field,message:fieldErrorMessage(field)});
		}

		function requestAction(action, params, callback, useSession){
			params = params || {};
			params.action = action;
			if(useSession !== false){
				params.sessionID = getSessionID();
				params.sessionUser = getSessionUserID();
			}
			params.callback = 'JSON_CALLBACK';
			var url = loginURL+'login/json.php'
			  , p = '';
			for(var k in params){
				p += (p ? '&' : '?') + k + '=' + encodeURI(params[k]);
			}
			$http.jsonp(url+p)
				.success(function(response){
					callback(response);
				})
				.error(function(){
					callback(false);
				});
			return instance;
		}

		function register(data, callback){
			var errors   = getDataErrors(data)
			, url        = loginURL+'login/add.php?action=addUser'
			, validParam = true
			, val        = null
			if(errors){
				setTimeout(function(){
					onFail(errors, callback);
				}, 1);
			} else {
				$http.post(url, data)
				.success(function(response){
					if(!response || response.error){
						onFail(response.error, callback);
					} else {
						if (response.sessionID) {
							response.session = {
								ID: response.sessionID,
								User: response.sessionUser
							}
						}
						saveSessionSQLite(response);
						callback(true);
					}
				})
				.error(function(){
					onFail('timeout', callback);
				});
			}
		}

		function whenLogged(callback){
			if(user){
				callback(user);
			} else {
				whenLoggedListeners.push(callback);
			}
			return user;
		}

		var instance = {
			action:requestAction,
			connect:connect,
			getSessionID:getSessionID,
			getSessionUserID:getSessionUserID,
			load:load,
			login:login,
			logoff:logoff,
			register:register,
			user:user,
			whenLogged:whenLogged,
			checkUserLogged:checkUserLogged
		};
		return instance;
	}]
)

.factory('ArtistModel' , ['$http', function($http) {
	var getSongCache = {}
	,   ArtistModel = {};

	function getSongInfo(artUrl, musUrl, cb) {
		if(getSongCache && getSongCache.artUrl === artUrl && getSongCache.musUrl === musUrl){
			setTimeout(function(){
				cb(getSongCache);
			}, 1);
		}
		$http
			.get('http://api.vagalume.com.br/search.php?art='+artUrl+'&mus='+musUrl)
			.success(function(data) {
				getSongCache = data || false;
				cb && cb(getSongCache);
			})
			.error(function(e) {
				cb && cb(false);
			});
	}

	ArtistModel.getSong = function(artUrl, musUrl, cb) {
		if(getSongCache && getSongCache.artUrl === artUrl && getSongCache.musUrl === musUrl){
			setTimeout(function(){
				cb(getSongCache);
			}, 1);
		} else {
			$http
				.get('http://api.vagalume.com.br/search.php?art='+artUrl+'&mus='+musUrl)
				.success(function(data) {
					if(data.type != 'song_notfound' && data.type != 'notfound') {
						var songText = data.mus[0].text
						,   allLines = songText.split(/\r\n|\r|\n/)
						,   lines    = [];

						for (var i = 0; i < allLines.length; i++) {
							if(allLines[i] != '') {
								lines.push(allLines[i]);
							}
						};

						var song = {
							artUrl   : artUrl,
							musUrl   : musUrl,
							artId    : data.art.id,
							musId    : data.mus[0].id,
							artName  : data.art.name,
							langId   : data.mus[0].lang,
							musName  : data.mus[0].name,
							text     : data.mus[0].text,
							lines    : data.mus[0].text.split('\n')
						}
						var words    = []
						,   newWords = [];

						for (var i = 0, ii = lines.length; i < ii; i++) {
							words = lines[i].split(/[\s,.;:?!¿¡*$#\"\(\)\[\]{}]/);
							for (var j = 0; j < words.length; j++) {
								if(words[j].length >= 3) {
									newWords.push(words[j]);
								}
							};
						};
						song.words = newWords;
						song.beginner = {
							words: {
								goal: Math.round(newWords.length * .10)
							},
							lines: {
								goal: Math.round(lines.length * .10)
							}
						}
						song.intermediate = {
							words: {
								goal: Math.round(newWords.length * .25)
							},
							lines: {
								goal: Math.round(lines.length * .25)
							}
						}
						song.advanced = {
							words: {
								goal: Math.round(newWords.length * .50)
							},
							lines: {
								goal: Math.round(lines.length * .50)
							}
						}
						song.expert = {
							words: {
								goal: newWords.length
							},
							lines: {
								goal: lines.length
							}
						}
					} else {
						var song = false;
					}

					getSongCache = song;
					cb(song);
				})
				.error(function(e) {
					cb(false);
				})
		}
	}

	ArtistModel.getVideos = function(pointerID, cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getVideos&pointerID='+pointerID)
		.success(function(res) {
			if(res.error == 0) {
				cb(res.data);
			}
		})
		.error(function(e) {
			cb(false);
		})
	}

	function Verse(verseData){
		this.verse = verseData.verse;
		this.start = verseData.startTime;
		this.end   = verseData.endTime;
		this.lines = verseData.lines;
		this.words = verseData.words;
		this.answers = [];
		for(var i=0, s=verseData.words.length; i<s; i++){
			this.answers.push(verseData.words[i][3]);
		}
	}

	ArtistModel.getSongSubtitle = function(duration, videoID, pointerID, level, mode, langID, sID, cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getSongSubtitle&pointerID='+pointerID+'&duration='+duration+'&videoID='+videoID+'&level='+level+'&mode='+mode+'&langID='+langID+(sID ? '&sID='+sID : ''))
		.success(function(res) {
			if(res.error == 0) {
				var data = res.data,
					subtitles = [];
				for(var i=0, s=data.subtitles.length; i<s; i++){
					subtitles.push(new Verse(data.subtitles[i]));
				}
				data.subtitles = subtitles;
				cb(data);
			} else {
				cb(false);
			}
		})
		.error(function(e) {
			cb(false);
		})
	}

	return ArtistModel;
}])

.factory('GameModel', ['$http', '$timeout', 'sessionUser', '$q', function($http, $timeout, sessionUser, $q){

	var b64 = {_keyStr:"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",encode:function(e){var t="";var n,r,i,s,o,u,a;var f=0;e=b64._utf8_encode(e);while(f<e.length){n=e.charCodeAt(f++);r=e.charCodeAt(f++);i=e.charCodeAt(f++);s=n>>2;o=(n&3)<<4|r>>4;u=(r&15)<<2|i>>6;a=i&63;if(isNaN(r)){u=a=64}else if(isNaN(i)){a=64}t=t+this._keyStr.charAt(s)+this._keyStr.charAt(o)+this._keyStr.charAt(u)+this._keyStr.charAt(a)}return t},decode:function(e){var t="";var n,r,i;var s,o,u,a;var f=0;e=e.replace(/[^A-Za-z0-9\+\/\=]/g,"");while(f<e.length){s=this._keyStr.indexOf(e.charAt(f++));o=this._keyStr.indexOf(e.charAt(f++));u=this._keyStr.indexOf(e.charAt(f++));a=this._keyStr.indexOf(e.charAt(f++));n=s<<2|o>>4;r=(o&15)<<4|u>>2;i=(u&3)<<6|a;t=t+String.fromCharCode(n);if(u!=64){t=t+String.fromCharCode(r)}if(a!=64){t=t+String.fromCharCode(i)}}t=b64._utf8_decode(t);return t},_utf8_encode:function(e){e=e.replace(/\r\n/g,"\n");var t="";for(var n=0;n<e.length;n++){var r=e.charCodeAt(n);if(r<128){t+=String.fromCharCode(r)}else if(r>127&&r<2048){t+=String.fromCharCode(r>>6|192);t+=String.fromCharCode(r&63|128)}else{t+=String.fromCharCode(r>>12|224);t+=String.fromCharCode(r>>6&63|128);t+=String.fromCharCode(r&63|128)}}return t},_utf8_decode:function(e){var t="";var n=0;var r=c1=c2=0;while(n<e.length){r=e.charCodeAt(n);if(r<128){t+=String.fromCharCode(r);n++}else if(r>191&&r<224){c2=e.charCodeAt(n+1);t+=String.fromCharCode((r&31)<<6|c2&63);n+=2}else{c2=e.charCodeAt(n+1);c3=e.charCodeAt(n+2);t+=String.fromCharCode((r&15)<<12|(c2&63)<<6|c3&63);n+=3}}return t}}

	function getLines(text){
		var lines1 = text.split(/\r\n|\r|\n/)
		  , lines2 = []
		  , line;
		for(var i=0, s=lines1.length; i<s; i++){
			line = lines1[i].trim();
			if(line) lines2.push(line);
		}
		return lines2;
	}

	function getWordCount(text){
		return text.split(' ').length;
	}

	function getLevelCode(level){
		switch(level) {
			case 1:
			case 'iniciante':
			case 'beginner':
				return 'beginner';
			case 2:
			case 'intermediario':
			case 'intermediate':
				return 'intermediate';
			case 3:
			case 'avancado':
			case 'advanced':
				return 'advanced';
			case 4:
			case 'expert':
				return 'expert';
		}
		return null;
	}

	function getSubtitlesOptions(pointerID, langID, duration, cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/subtitle-get.php?action=getSubtitlesForLang&pointerID='+pointerID+'&duration='+duration+'&langID='+langID)
		.success(function(data) {
			if(data.subtitles){
				cb && cb(data);
			} else {
				cb && cb(false);	
			}
		})
		.error(function(e) {
			cb && cb(false);
		});
	}

	function denunciaSubtitle(pointerID, sID, cb) {
		$http
		.get('https://app2.vagalume.com.br/ajax/subtitle-post.php?action=denunciaSubtitle&pointerID='+pointerID+'&subtitleID='+sID)
		.success(function(data) {
			if(data){
				cb && cb({'status': data[0], 'message': data[1]});
			} else {
				cb && cb(false);	
			}
		})
		.error(function(e) {
			cb && cb(false);
		});
	}

	function getLevelSlug(level){
		switch(level){
			case 1:
			case 'iniciante':
			case 'beginner':
				return 'iniciante';
			case 2:
			case 'intermediario':
			case 'intermediate':
				return 'intermediario';
			case 3:
			case 'avancado':
			case 'advanced':
				return 'avancado';
			case 4:
			case 'expert':
				return 'expert';
		}
		return '';
	}

	function getTypeSlug(type){
		switch(type){
			case 1: return 'video';
			case 2: return 'frase';
		}
		return '';
	}

	function getLangSlug(lang){
		switch(lang){
			case 2: return 'ingles';
			case 3: return 'espanhol';
		}
		return '';
	}

	function getLangID(lang){
		switch(lang){
			case 2:
			case 'inglês':
			case 'ingles':
				return 2;
			case 3:
			case 'espanhol':
				return 3;
		}
		return 0;
	}

	function getRandomSongs(mode, cb) {
		mode = mode || '';
		switch(mode){
			case 'frase':
			case 'phrase':
				mode = 'verse';
		}
		var defer = $q.defer()
		  , lang  = (['eng', 'spa'])[Date.now() % 2];

		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getRandomSong&mode='+mode+'&lang='+lang)
		.success(function(res) {
			if(res.data){
				var data = res
					.data
					.sort(function(){
						return (Math.random() * 2) - 1;
					})
					.slice(0, 5);
				cb && cb(res.data);
				defer.resolve(res.data);
			} else {
				cb && cb(false);
				defer.reject(false);
			}
		})
		.error(function(e) {
			cb && cb(false);
			defer.reject(false);
		});
		return defer.promise;
	}

	function getSong(artUrl, musUrl,  cb, related) {
		var k = artUrl + '/' + musUrl;
		if(getSong.currentSong && getSong.currentSong[k]){
			cb(getSong.currentSong[k]);
			return getSong.currentSong[k];
		}
		var info = (getSong.currentSong = {})[k] = {};;
		$http
			.get('http://api.vagalume.com.br/search.php?art='+artUrl+'&mus='+musUrl+(related ? '&extra=relmus' : ''))
			.success(function(data) {
				if(data){
					for(var k in data){
						info[k] = data[k];
					}
				}
				cb && cb(info);
			})
			.error(function(e) {
				cb && cb(false);
			});
		return info;
	}

	function getRandomVerses(level, langID, cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getRandomVerses&level='+level+'&langID='+langID)
		.success(function(res) {
			if(res.error == 0) {
				var data = res.data;
				cb(data);
			}
		})
		.error(function(e) {
			cb(false);
		});
	}

	function getGameRank(pointerID, gameMode, cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getGameRank&pointerID='+pointerID+'&gameMode='+gameMode)
		.success(function(res) {
			if(res.error == 0) {
				var data = res.data;
				cb(data);
			}
		})
		.error(function(e) {
			cb(false);
		});
	}

	function getUserScore(login, cb) {
		if(login){
			$http
			.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getUserScore&login=' + login)
			.success(function(res) {
				if(res.error == 0) {
					var data = res.data;
					cb(data);
				}
			})
			.error(function(e) {
				cb(false);
			});
		} else {
			$timeout(cb, 1);
		}
	}

	// Modo frase
	function getVerses (artUrl, musUrl, level, cb) {
		$http
		.get('http://api.vagalume.com.br/search.php?art='+artUrl+'&mus='+musUrl)
		.success(function(data) {
			var origID     = data.mus[0].id,
				bandID     = data.art.id,
				origTitle  = data.mus[0].name,
				artist     = data.art.name,
				song       = data.mus[0].name;

			$http.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getVerses&origID='+origID+'&bandID='+bandID+'&origTitle='+origTitle+'&level='+level)
			.success(function(res) {
				if(res.error == 0) {
					var data = res.data;

					cb({
						song: song,
						pointerID: origID,
						artist: artist,
						verses: data.verses,
						lyrics: data.lyrics
					});
				} else {
					cb(false);
				}
			})
		})
		.error(function(e) {
			cb(false);
		})
	}

	function terminate(incScore, pointerID, gameMode, cb) {
		var sessionUserID = sessionUser.getSessionUserID()
		,   sessionID   = sessionUser.getSessionID();

		if(sessionID && sessionUserID && incScore) {
			$http
			.post('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=terminate', {
				l: sessionID,
				m: sessionUserID,
				v: b64.encode('{"v":"'+incScore+'","p":"'+pointerID+'", "m":"'+gameMode+'"}'),
				h: 'iownTewngit73h721895jdbhfjwR3H3'
			})
			.success(function(res) {
				if(res.error == 0) {
					var data = res.data;
					cb(data);
				}
			})
			.error(function(e) {
				cb(false);
			});
		} else if (pointerID) {
			$http
			.post('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=terminate', {
				p: b64.encode('{"p":"'+pointerID+'"}'),
				h: 'iownTewngit73h721895jdbhflvebjwR3H3'
			})
			.success(function(res) {
				if(res.error == 0) {
					var data = res.data;
					cb(data);
				}
			})
			.error(function(e) {
				cb(false);
			});
		}
	}

	getSong.currentSong = {};

	var timer = {
		left:0,
		id:null,
		on:null,
		loop:function(){
			timer.left = Math.max(timer.left - 1, 0);
			if(timer.left) {
				timer.fire('tick');
				timer.id = $timeout(timer.loop, 1000);
			} else {
				timer.id = null;
				timer.fire('end');
			}
		},
		fire:function(type){
			this.on && this.on.call(this, type);
		},
		play:function(left){
			left = (left === 0) ? 0 : (left || this.left);
			if(left){
				this.left = left;
				if(!this.id){
					this.fire('play');
					this.id = $timeout(timer.loop, 1000);
				}
			} else {
				this.fire('stop');
			}
		},
		stop:function(){
			if(timer.id){
				$timeout.cancel(timer.id);
				timer.id = null;
			}
		}
	};

	return {
		getWordCount       : getWordCount,
		getLines           : getLines,
		getLevelCode       : getLevelCode,
		getLevelSlug       : getLevelSlug,
		getTypeSlug        : getTypeSlug,
		getLangID          : getLangID,
		getLangSlug        : getLangSlug,
		getSong            : getSong,
		getRandomVerses    : getRandomVerses,
		getRandomSongs     : getRandomSongs,
		denunciaSubtitle   : denunciaSubtitle,
		getSubtitlesOptions: getSubtitlesOptions,
		getTimer           : function(){return timer;},
		getVerses          : getVerses,
		getGameRank        : getGameRank,
		getUserScore       : getUserScore,
		terminate          : terminate
	};
}])

.factory('GameScore', [function(){

	function GameScore(){
		this.init();
	};

	function border(min, val, max){
		return Math.max(min, Math.min(val, max));
	}

	GameScore.prototype = {
		fail:function(){
			this.failCount++;
			return this;
		},
		skip:function(){
			this.failCount++;
			return this.decChance();
		},
		hit:function(){
			this.hitCount++;
			return this.decChance();
		},
		decChance:function(){
			this.chancesLeft = border(0, this.chancesLeft - 1, this.chancesTotal);
			return this;
		},
		init:function(){
			this.score = 0;
			this.timeLeft = 100;
			this.failCount = 0;
			this.hitCount = 0;
			this.total = 0;
			this.chancesLeft = 0;
			this.chancesTotal = 0;
			return this;
		},
		setChances:function(count){
			this.chancesLeft = this.chancesTotal = count;
			return this;
		},
		incTime:function(inc){
			this.timeLeft = border(0, this.timeLeft + inc, 100);
			return this;
		},
		hasEnded:function(){
			return this.chancesLeft == 0 || this.timeLeft == 0;
		}
	};

	return {
		create:function(callback){
			return new GameScore(callback);
		}
	}
}])

.factory('GamePhrase', ['GameModel', 'GameScore', '$q', '$timeout', 'sessionUser', function(GameModel, GameScore, $q, $timeout, sessionUser){
	var instance = null
	  , HIT_SCORE = 15
	  , TIME_DEC  = 5;

	function sortSuggests(a, b){
		return a.id - b.id;
	}

	function shuffle(){
		return (Math.random() * 2) - 1;
	}

	function copyArray(list1){
		var list2 = [];
		for(var i=0; i < list1.length; i++) list2.push(list1[i]);
		return list2;
	}

	function addMoreSuggest(list, index) {		
		var versesCount  = this.verseList.length
		,   suggestArray = [];
		if(versesCount > 1) {
			for (var i = 0; i < this.verseList.length; i++) {
				if(this.verseList[index].orig != this.verseList[i].orig) {
					var words = copyArray(this.verseList[i].answerList).sort(shuffle);
					for (var j = 0; j < words.length; j++) {
						suggestArray.push(words[j].text);
					}
				}				
			};
			suggestArray.sort(shuffle);
			for (var i = 0; i < suggestArray.length; i++) {
				var notValid = false;
				for (var j = 0; j < this.verseList[index].answerList.length; j++) {
					if(this.verseList[index].answerList[j].text == suggestArray[i]) {
						notValid = i;
					}
				};
				if(notValid) {
					suggestArray.slice(notValid, 1);
				}
			};

			if(suggestArray) {
				if(suggestArray.length > 4) {
					suggestArray = suggestArray.sort(shuffle).slice(0, 3);
				}
				var id = list.length - 1;
				for (var i = 0; i < suggestArray.length; i++) {
					id++;
					list.push({
						id: id,
						text: suggestArray[i]
					})
				};
			}
		}
		return list;
	}

	function isEqual(a, b){
		if(a && b && a.length == b.length && a.length){
			var i = a.length;
			while(i--){
				if(a[i].text !== b[i].text) return false;
			}
			return true;
		}
		return false;
	}

	function updateUserSuggestList(){
		this.verseList[this.verseIndex].userList = this.current.userList;
	}

	function isRight(){
		if(this.current){
			return isEqual(this.current.answerList, this.current.userList);
		}
		return false;
	}

	function GamePhrase(bandSlug, songSlug, level){
		this.bandSlug = bandSlug;
		this.songSlug = songSlug;
		this.level    = GameModel.getLevelCode(level);
		this.timeLeft;
		var self = this;
		this.score = GameScore.create();
		(this.timer = GameModel.getTimer()).on = function(type){
			self.timeLeft = this.left;
			switch(type){
				case 'end':
					self.decreaseProgress();
			}
		};
	}

	function endGame() {
		var self = this;		
		this.on && this.on('end');
	}

	GamePhrase.prototype = {
		run:function(){
			var self = this
			  , defer = $q.defer();
			GameModel.getVerses(self.bandSlug, self.songSlug, self.level, function(data) {
				if(data && !data.error){
					self.bandName = data.artist;
					self.songName = data.song;
					self.songID   = data.pointerID;
					self.score.score = 0;
					self.lyrics   = data.lyrics;
					var verses = data.verses
					  , verse
					  , suggests
					  , suggest
					  , verseList = []
					  , id = 1
					  , llen = self.lyrics.length
					  , lyricsIndex;
					for(var i=0, iLen=verses.length; i<iLen; i++){

						verse = verses[i];

						var verseAnswerList  = verse.suggests.sort(sortSuggests)
						  , word;

						for(var j=0, jLen = verseAnswerList.length; j < jLen; j++){
							verseAnswerList[j] = {
								id:id++,
								text:verseAnswerList[j].word
							}
						}

						for(var k=0; k < llen; k++){
							if(verse.verse == self.lyrics[k]){
								lyricsIndex = k;
							}
						}

						verseList.push({
							orig       :verse.verse,
							before     :verse.before,
							after      :verse.after,
							answerList :verseAnswerList,
							userList   :[],
							lyricsIndex:lyricsIndex
						});

					}

					self.verseList  = verseList;
					self.score.setChances(verseList.length).timeLeft = 100;
					self.verseIndex = -1
					self.nextVerseIndex = 0;
					self.getNextPhrase('start');
					defer.resolve(self);
				} else {
					defer.reject(data && data.error || 'error');
				}
			});
			return defer.promise;
		},
		getPhrase:function(index){
			var item = this.verseList[index];
			if(item){
				return {
					answerList:item.answerList,
					suggestList:addMoreSuggest.call(this, copyArray(item.answerList), index).sort(shuffle),
					userList:[],
					index:index,
					orig:item.orig,
					before:item.before,
					after:item.after,
					lyricsIndex:item.lyricsIndex
				};
			}
			return null;
		},
		decreaseTimer: null,
		decreaseProgress:function() {
			if(this.decreaseProgress && !this.isPaused){
				var self = this;
				this.decreaseTimer = $timeout(function() {
					self.decreaseTimer = null;
					if(!self.isPaused){
						if(self.score.incTime(-1).timeLeft <= 0) {
							endGame.call(self);
						} else {
							self.decreaseProgress();
						}
					}
				}, 1000);
			}
			return this;
		},
		pause:function(){
			this.isPaused = true;
			if(this.decreaseTimer){
				$timeout.cancel(this.decreaseTimer);
				this.decreaseTimer = null;
			}
			return this;
		},
		unpause:function(){
			delete this.isPaused;
			return this.decreaseProgress();
		},
		getNextPhrase:function(type){
			var next = true;
			type = type || 'jump';
			if(this.verseIndex < this.score.chancesTotal && this.score.chancesLeft > 0){
				switch(type){
					case 'score':
						if(isRight.call(this)){
							this.score.incTime(1).hit().score += HIT_SCORE;
						} else {
							if(this.score.fail().incTime(-TIME_DEC).timeLeft <= 0) {
								endGame.call(this);
							}
							next = false;
						}
						break;
					case 'jump':
						if(this.score.skip().incTime(-TIME_DEC).timeLeft <= 0) {
							endGame.call(this);
						}
						break;
					case 'start':
				}
				if(next) {
					$timeout.cancel(this.decreaseTimer);
					this.timer.play(30);
				}
				if((this.verseIndex + 1) < this.score.chancesTotal){
					if(!this.score.hasEnded()){
						if(next) {
							this.current = this.getPhrase(this.verseIndex = this.nextVerseIndex++);
							this.on && this.on('changeVerse');
						} else {
							this.on && this.on('wrongAnswer');
						}
					}
					return true;
				}
			}
			if(!this.ended){
				this.ended = true;
				endGame.call(this);
			}
			return false;
		},
		addSuggest:function(index){
			var current = this.current;
			if(current.suggestList[index]){
				var newList = [];
				for(var i=0, s=current.suggestList.length; i<s; i++){
					if(i == index){
						current.userList.push(current.suggestList[i]);
					} else {
						newList.push(current.suggestList[i]);
					}
				}
				current.suggestList = newList;
				updateUserSuggestList.call(this);
			}
		},
		removeSuggest:function(index){
			var userList = this.current.userList
			  , current = this.current;
			if(userList[index]){
				var newList = [];
				for(var i=0, s=userList.length; i<s; i++){
					if(i == index){
						current.suggestList.push(userList[i]);
					} else {
						newList.push(userList[i]);
					}
				}
				current.userList = newList;
				updateUserSuggestList.call(this);
			}
		}
	}

	return {
		get:function(band, song){
			if(arguments.length == 2){
				var songs = map[band] = (map[band] || {});
				return instance = songs[song] = songs[song] || new GamePhrase(band, song);
			}
			return instance;
		},
		run:function(bandSlug, songSlug, level){
			return (instance = new GamePhrase(bandSlug, songSlug, level)).run();
		}
	};
}])

.factory(
	'GameVideo',
	['$interval', 'sessionUser', '$q', '$rootScope', 'ArtistModel', 'GameModel', 'GameScore', 'VideoModel', '$timeout', 'YoutubeVagalumePlayer', 'postMessenger',
	function($interval, sessionUser, $q, $rootScope, ArtistModel, GameModel, GameScore, VideoModel, $timeout, YoutubeVagalumePlayer, postMessenger){
		var map              = {}
		  , loopInstance     = null
		  , instance         = null
		  , intervalPromise  = null
		  , PERCENT_DECREASE = -5;

		$rootScope.$on('vagalume.player.timescan', function($event, vagalumePlayer, time){
			if(loopInstance && loopInstance.vagalumePlayer){
				if(loopInstance.gamePlaying){
					var vagalumePlayer = loopInstance.vagalumePlayer
					  , youtubePlayer = vagalumePlayer.youtubePlayer;
					switch(vagalumePlayer.currentState){
						case 'playing':
							if(loopInstance.gamePlaying){
								onTimeUpdate.call(loopInstance, time || 0);
							}
						default:
							break;
						case 'ended':
							stopLoop();
					}
				}
			} else {
				stopLoop();
			}
		});

		function startLoop(instance){
			loopInstance = instance;
			return instance;
		}

		function stopLoop(){
			loopInstance = null;
		}

		function onTimeUpdate(time){
			if(this.videoPlaying){
				
				var useArgs = true;
				switch(this.loopMode){
					case 'user-repeat':
					case 'end-repeat':
						useArgs = false;
				}
				if(useArgs && this.changedArgs){
					onSubtitleChange.apply(this, this.changedArgs);
				} else {
					if(this.subtitle){
						time += parseInt(this.subtitle.offset);
						var list = this.subtitle.subtitles
						  , item = null;
						for(var i=0, ii=list.length; i < ii; i++){
							item = list[i];
							if(item.start <= time && time < item.end){
								if(this.currentVerseIndex != i){
									return onPrepareSubtitleChange.call(this, this.currentVerseIndex, i, item);
								}
								return;
							}
						}
						if(this.currentVerseIndex != -1){
							return onPrepareSubtitleChange.call(this, this.currentVerseIndex, -1, null);
						}
					}
				}
			}
		}

		function onPrepareSubtitleChange(prev, next, nextItem){
			switch(this.loopMode){
				case 'end-repeat':
					this.loopMode = 'normal';
					return onSubtitleChange.apply(this, arguments);
			}
			this.changedArgs = arguments;
			if(this.currentItem && this.currentItem.words.length){
				if(this.answersLeft){
					this.loopMode = 'end-repeat';
					this.vagalumePlayer.pause(true);
					this.answerWait = true;
					this.timer.play(101);
					this.videoPlaying = false;
				}
			}
		}

		function onSubtitleChange(prev, next, nextItem){
			if(this.currentItem && this.currentItem.words.length && this.answersLeft){
				return;
			}
			clearTimeout(this.pauseTimeout);
			var nextVerse = this.subtitle.subtitles[this.currentVerseIndex+1];
			this.changedArgs = null;
			this.prevVerseIndex    = this.currentVerseIndex;
			this.prevVerseText     = this.currentVerseText;
			this.prevItem          = this.currentItem;
			this.currentVerseIndex = next;
			this.currentVerseText  = nextItem && nextItem.verse || '';
			this.currentItem       = nextItem;
			this.answerVerseText   = this.currentVerseText;
			if(nextVerse) {
				this.nextVerseText = nextItem && nextVerse.verse || '';
			} else {
				this.nextVerseText = '';
			}

			animateShadow(this);

			if(this.currentItem && this.currentItem.words.length){
				this.answersMatrixIndex = 0;
				this.answersMatrix = this.currentItem.words;
				this.answersLeft = this.answersMatrix.length;
				this.progressStep++;
				$rootScope.$emit('subtitleChangeOptions', this, true);
			} else {
				$rootScope.$emit('subtitleChange', this);
			}
			$timeout(function(){});
		}

		function animateShadow(game) {
			if (game.currentVerseText != '') {
				game.currentItem.style = {
					'-webkit-transition': 'none',
					'-moz-transition': 'none',
					'-o-transition': 'none',
					'-ms-transition': 'none'
				}

				$timeout(function() {
					game.currentItem.style = {
						'width': '0'
					}
					$timeout(function() {
						var time = game.currentItem.end - game.currentItem.start;
						time = 'width '+time+'s linear';
						game.currentItem.style = {
							'-webkit-transition': time,
							'-moz-transition': time,
							'-o-transition': time,
							'-ms-transition': time,
							'width': '100%'
						}
					}, 10);
				}, 10);
			}
		}

		$rootScope.$on('vagalume.player.playing', function(){
			if(loopInstance){
				if(!loopInstance.gameStarted){
					loopInstance.gameStarted = loopInstance.gamePlaying = true;
					$rootScope.$emit('start', loopInstance);
				}
				loopInstance.videoPlaying = true;
				startLoop(loopInstance);
				loopInstance.timer.stop();
				$rootScope.$emit('subtitleChange', loopInstance);
			}
		});

		$rootScope.$on('vagalume.player.ended', function() {
			stopLoop();
			$rootScope.$emit('endVideo', instance);
		});
		$rootScope.$on('vagalume.player.buffering', function() {
			$rootScope.$emit('loadingVideo');
		});

		function onPause(){
			if(loopInstance) {
				if(loopInstance.gamePlaying){
					loopInstance.videoPlaying = false;
					loopInstance.timer.play();
				}
			}
		}

		$rootScope.$on('vagalume.player.paused', onPause);
		$rootScope.$on('vagalume.player.pausing', onPause);

		function VideoGame(band, song){
			this.bandSlug = band;
			this.songSlug = song;
			this.progressTotal = 0;
			this.timeLeft = 0;
			this.score = 0;
			this.progressDone = 0;
			this.progressTotal = 0;
			this.started = false;
			this.score = GameScore.create();
			var self = this;
			(this.timer = GameModel.getTimer()).on = function(type){
				self.timeLeft = this.left;
				if(!self.gameEnded) {
					switch(type){
						case 'tick':
							if(self.updateTimeLeft(-1) <= 0) {
								this.left = 0;
								self.endGame('timeEnd');
								self.timer.stop();
								self.timer = null;
							}
							break;
					}					
				} else {
					if(self.timer) {
						self.timer.stop();						
					}
					self.timer = null;
				}
			};
		}

		VideoGame.prototype = {			
			endGame:function(reason){
				this.endReason = reason;
				switch(reason) {
					case 'timeEnd':
						this.vagalumePlayer.pause();
						stopLoop();
						break;
					case 'blankEnd':
						break;
				}
				this.gameEnded = true;
				$rootScope.$emit('endGame', this, reason);
				this.endGame = function(){};
			},
			updateTimeLeft:function(inc){
				this.score.incTime(inc);
				if(this.score.timeLeft <= 0) this.endGame('timeEnd');
				return this;
			},
			pause:function(enableTransition){
				this.gamePlaying = false;
				this.timer.stop();
				stopLoop();
				this.vagalumePlayer.pause(enableTransition);
				return this;	
			},
			unpause:function(){
				if(this.score.chancesLeft > 0){
					if(this.videoPlaying && !this.answerWait){
						this.vagalumePlayer.play();
						startLoop(this);
					}
					this.gamePlaying = true;
					this.timer.play();
				}
				return this;
			},
			replay:function() {
				if(this.endReason == 'timeEnd') return;
				this.loopMode = 'end-repeat';
				switch(this.vagalumePlayer.currentState){
					case 'paused':
					case 'pausing':
						var self = this;
						$timeout(function() {
							self.vagalumePlayer.play();
							animateShadow(self);
						}, 200);
						this.vagalumePlayer.seekTo(this.currentItem.start);
				}
			},
			loadVideo:function(){
				var self = this
				  , defer = $q.defer();
				if(this.videoId){
					defer.resolve(this.videoId);
				} else {
					ArtistModel.getSong(this.bandSlug, this.songSlug, function(data){
						if(data && data.artUrl){
							self.songLangId = data.langId;
							self.songId     = data.musId;
							self.bandName   = data.artName;
							self.songName   = data.musName;
							self.bandId     = data.musId;
							self.text       = data.text;
							self.wordCount  = GameModel.getWordCount(data.text);
							self.lineCount  = GameModel.getLines(data.text).length;
							ArtistModel.getVideos(self.songId, function(videoList){
								self.videoList = videoList;
								self.allVideoList = [];
								for (var i = 0; i < videoList.length; i++) {
									self.allVideoList.push(videoList[i]);
								}
								if ($rootScope.videoId) {
									self.videoId = $rootScope.videoId;
									defer.resolve($rootScope.videoId)
								} else {
									if(videoList && videoList.length){
										VideoModel.getFirstValidVideoID(videoList, function(videoId, error){
											if(videoId){
												self.videoId = videoId;
												defer.resolve(videoId);
											} else {
												defer.reject(error);
											}
										});
									} else {
										defer.reject('no video');
									}
								}
							});
						} else {
							defer.reject('no song');
						}
					});
				}
				return defer.promise;
			},
			loadSubtitles:function(level, sID){
				this.currentVerseIndex = -2;
				this.currentVerseText = '...';
				this.prevVerseIndex = -2;
				this.waitAnswer = false;
				this.vagalumePlayer = YoutubeVagalumePlayer.create();
				this.optionsIndex = 0;
				this.anwsers = null;
				startLoop(this);
				var self = this
				  , player = this.vagalumePlayer.youtubePlayer
				  , defer = $q.defer();
				this.loadVideo().then(
					function(videoId){
						postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'getDuration')
						.then(function(duration) {
							ArtistModel.getSongSubtitle(duration, videoId, self.songId, level, 'video', self.songLangId, sID, function(data) {
								if(data){
									self.subtitle = data;
									self.sID = data.sID;
									var wordsCount = 0;
									for (var i = 0; i < data.subtitles.length; i++) {
										wordsCount += data.subtitles[i].words.length;
									};
									self.score = GameScore.create().setChances(self.progressTotal = wordsCount);
									self.duration = duration;
									self.progressStep = 0;
									self.progressDone = 0;
									self.progressSum = 1;
									self.goal = wordsCount;
									self.subtitleListLen = (data.subtitles || []).length
									defer.resolve(data);
								} else {
									defer.reject('no subtitle');
								}
							});							
						});
					},
					function(error){
						defer.reject(error);
					}
				);
				return defer.promise;
			},
			stopLoop:function(){
				stopLoop();
				return this;
			},
			repeat:function(){
				if(this.currentItem && this.currentItem.words.length) {
					this.timer.stop();
					this.updateTimeLeft(PERCENT_DECREASE);
					this.loopMode = 'user-repeat';
					this.vagalumePlayer.seekTo(this.currentItem.start);
					this.vagalumePlayer.play();
					startLoop(this);
					animateShadow(this);
				}
			},
			setAnswer:function(text, skip){
				var hit = false;
				skip = !text;
				// Se existe lacunas para preencher
				if(this.answersLeft > 0){
					// Se tiver palavras para preencher ainda
					if(this.score.chancesLeft > 0) {
						var list = this.answersMatrix && this.answersMatrix[this.answersMatrixIndex] || null;
						// Acertou
						if(list instanceof Array && (list[3] == text)){
							// Aumenta 1% do tempo do jogo, pois o usuário acertou
							this.updateTimeLeft(1);
							// Aumenta o score em 5 pontos
							// Aumenta os acertos do usuário
							this.score.hit().score += 5;
							// Diminui o número de lacunas, pois essa já foi preenchida
							this.answersLeft--;
							hit = true;
						} else {
							// Errou
							this.updateTimeLeft(PERCENT_DECREASE);
							this.score.fail();
						}
						// Pulou
						if(skip) {
							this.score.decChance();
							if(this.score.timeLeft > 0) {
								this.answersLeft--;
								this.score.incTime(-1);
							}
						}
					}
					var hasEnded = !(this.answersMatrixIndex + 1 < this.answersMatrix.length);
					if(skip) {
						$rootScope.$emit('computeAnswer', text, 'skip', hasEnded);
						var subtitle      = this.subtitle.subtitles[this.currentVerseIndex]
						,   words         = subtitle.words
						,   firstVerifier = subtitle.verse;
						subtitle.verse = subtitle.verse.replace('<b>____</b>', '<strong class=jump>'+words[this.answersMatrixIndex][3]+'</strong>');
						if(firstVerifier.substring(0, 2) == '<b') {
							subtitle.verse = subtitle.verse.replace('<strong class=jump>'+words[this.answersMatrixIndex][3]+'</strong>', '<strong class="jump first">'+words[this.answersMatrixIndex][3]+'</strong>');
						}
						this.answerVerseText = subtitle.verse;
					} else if (hit) {
						$rootScope.$emit('computeAnswer', text, 'hit', hasEnded);
						var subtitle      = this.subtitle.subtitles[this.currentVerseIndex]
						,   firstVerifier = subtitle.verse;
						subtitle.verse = subtitle.verse.replace('<b>____</b>', '<strong class=correct>'+text+'</strong>');
						if(firstVerifier.substring(0, 2) == '<b') {
							subtitle.verse = subtitle.verse.replace('<strong class=correct>'+text+'</strong>', '<strong class="correct first">'+text+'</strong>');
						}
						this.answerVerseText = subtitle.verse;
					}
					if (hit || skip) {
						this.progressDone++;
						if(this.answersMatrixIndex + 1 < this.answersMatrix.length){
							this.answerWait = true;
						} else {
							this.answerWait = !(this.answersDone = true);
							this.timer.stop();
							var subtitle = this.subtitle.subtitles[this.currentVerseIndex]
							,   words = subtitle.words;
							for (var i = 0; i < words.length; i++) {
								subtitle.verse = subtitle.verse.replace('<b>____</b>', words[i][3]);
							};
							this.videoPlaying = true;
						}
					}
					if (hit || skip) {
						this.answersMatrix.splice(0, 1);
						this.progressStep++;
						$rootScope.$emit('subtitleChangeOptions', this, false);
					}
					if(this.score.chancesLeft == 0){
						this.endGame('blankEnd');
					}
				}
				return hit;
			},
			getAnswerOptions:function(){
				if(this.answersMatrix){
					var list = this.answersMatrix && this.answersMatrix[this.answersMatrixIndex] || null;
					if(list instanceof Array){
						for(var i=list.length, options = []; i--;){
							options.push(list[i]);
						}
						return options.sort(function(){return (Math.random() * 2) - 1});
					}
				}
				return [];
			}
		}

		$rootScope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams){
			if(instance && instance.timer) {
				instance.timer.stop();
				instance.timer = null;
			}
		});

		return {
			get:function(band, song){
				if(arguments.length == 2){
					return instance = new VideoGame(band, song);
				}
				return instance;
			},
			stopLoop:stopLoop
		};
	}]
)

.factory('RankModel', ['$http', function($http){

	function getHotSpots(cb) {
		var list = getHotSpots.cache || [];
		if(!getHotSpots.cache){
			$http
				.get('http://api.vagalume.com.br/hotspots.php')
				.success(function(data) {
					if(data) {
						for (var i = 0; i < data.hotspots.length; i++) {
							if(data.hotspots[i].type == 'music') {
								list.push(data.hotspots[i]);
							}
						};
					}
					getHotSpots.cache = list;
					cb && cb(list);
				})
				.error(function(e) {
					cb && cb(false);
				});
		} else {
			cb && cb(list);
		}
		return list;
	}
	// Pega as músicas de destaque da home do Vagalume
	function getDestaques(cb) {
		var list = getDestaques.cache || [];
		if(!getDestaques.cache){
			$http
				.get('http://app2.vagalume.com.br/ajax/homespot.php')
				.success(function(data) {
					if(data instanceof Array){
						list = getDestaques.cache = data;
					}
					cb && cb(data);
				})
				.error(function(e) {
					cb && cb(false);
				});
		} else {
			cb && cb(list);
		}
		return list;
	}
	// Pega o rank de músicas mensal
	function getTopSongs(cb) {
		var list = getTopSongs.cache || [];
		if(!getTopSongs.cache){
			$http
				.get('http://api.vagalume.com.br/rank.php?type=mus&period=month&scope=lyrics&limit=30&limit=16')
				.success(function(data) {
					var list = [];
					if(data.mus.month.lyrics) {
						var lyrics = data.mus.month.lyrics
						  , regex = /([^/]+)\/([^/]+)\.html$/
						  , item;
						for (var i = 0, s = lyrics.length; i<s; i++) {
							item = lyrics[i];
							if(item){
								var urls = regex.exec(item.url);
								item.descUrl = {
									artUrl: urls && urls[1] || '',
									musUrl: urls && urls[2] || ''
								}
								list.push(item);
							}
						};
					}
					getTopSongs.cache = list;
					cb && cb(list);
				})
				.error(function(e) {
					cb && cb(false);
				});
		} else {
			cb && cb(list);
		}
		return list;
	}
	// Pega o rank de todos os meses do ano
	function getYearMonthRank(year, cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getYearMonthRank&year='+year)
		.success(function(res) {
			if(res.error == 0) {
				var data = res.data;
				cb(data);
			}
		})
		.error(function(e) {
			cb(false);
		})
	}
	// Pega o rank geral
	function getGeneralRank(cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getGeneralRank')
		.success(function(res) {
			if(res.error == 0) {
				var data = res.data;
				cb(data);
			}
		})
		.error(function(e) {
			cb(false);
		})	
	}
	// Pega o rank de todas as semanas do ano
	function getYearWeekRank(year, cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getYearWeekRank&year='+year)
		.success(function(res) {
			if(res.error == 0) {
				var data = res.data;
				cb(data);
			}
		})
		.error(function(e) {
			cb(false);
		})
	}

	function getMostPlayed(cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getMostPlayedSongs&limit=15')
		.success(function(res) {
			if(res.error == 0) {
				var data = res.data;
				cb(data);
			}
		})
		.error(function(e) {
			cb(false);
		})	
	}

	function getRecentPlayed(cb) {
		$http
		.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getRecentPlayedSongs&limit=15')
		.success(function(res) {
			if(res.error == 0) {
				var data = res.data;
				cb(data);
			}
		})
		.error(function(e) {
			cb(false);
		})	
	}

	return {
		getTopSongs:getTopSongs,
		getHotSpots:getHotSpots,
		getDestaques:getDestaques,
		getYearMonthRank:getYearMonthRank,
		getGeneralRank:getGeneralRank,
		getYearWeekRank:getYearWeekRank,
		getMostPlayed:getMostPlayed,
		getRecentPlayed:getRecentPlayed
	};
}])

.factory('GoogleAnalytics', function(){
	(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)})(window,document,'script','//www.google-analytics.com/analytics.js','ga');
	ga('create', 'UA-2854418-1', 'auto');
	analyticsPageView();
	function analyticsEvents(category,action,label){
		if(label) {
			ga('send', 'event',category,action,label);
		} else {
			ga('send', 'event',category,action);
		}
	}
	function analyticsPageView(obj){
		if(obj) {
			ga('send', 'pageview', obj);
		} else {
			ga('send', 'pageview');
		}
	}
	return {
		analyticsEvents:analyticsEvents,
		analyticsPageView:analyticsPageView
	}
})

.factory('RequestModel' , ['$http', function($http) {
	// Pega os últimos Hot Spots
	var getHotSpots = function(cb) {
		$http
			.get('http://api.vagalume.com.br/hotspots.php')
			.success(function(data) {
				if(data) {
					var filter = [];
					for (var i = 0; i < data.hotspots.length; i++) {
						if(data.hotspots[i].type == 'music') {
							filter.push(data.hotspots[i]);						
						}
					};
					cb(filter);					
				}
			})
			.error(function(e) {
				cb(false);
			})
	}
	// Pega as músicas de destaque da home do Vagalume
	,   getDestaques = function(cb) {
		$http
			.get('http://app2.vagalume.com.br/ajax/homespot.php')
			.success(function(data) {
				cb(data);
			})
			.error(function(e) {
				cb(false);
			})
	}
	// Pega o rank de músicas mensal
	,   getTopSongs = function(cb) {
		$http
			.get('http://api.vagalume.com.br/rank.php?type=mus&period=month&scope=lyrics&limit=15')
			.success(function(data) {
				if(data.mus.month.lyrics) {
					for (var i = 0; i < data.mus.month.lyrics.length; i++) {
						var regex = /([^/]+)\/([^/]+)\.html$/g
						,   urls = regex.exec(data.mus.month.lyrics[i].url);
						
						data.mus.month.lyrics[i].descUrl = {
							artUrl: urls[1],
							musUrl: urls[2]
						}
					};
					cb(data);					
				}
			})
			.error(function(e) {
				cb(false);
			})
	}
	// Pega as músicas de destaque dos idiomas
	,   getLangVideos = function(lang, cb) {
		$http
			.get('http://app2.vagalume.com.br/ajax/aprendaAPI.php?action=getLangVideos&lang='+lang)
			.success(function(response) {
				if (response.error === 0) {
					cb(response.data);					
				}
			})
			.error(function(e) {
				cb(false);
			})
	}
	,   doSuggest = function(q, cb, lang) {
		$http
			.get('http://suggest.vagalume.com.br/mus.json?limit=15&showscore=1&q='+q+(lang ? '&langID=' + lang : ''))
			.success(function(data) {
				cb(data);
			})
			.error(function(e) {
				cb(false);
			})
	}
	,   getSong = function(artUrl, musUrl, cb) {
		$http
			.get('http://api.vagalume.com.br/search.php?art='+artUrl+'&mus='+musUrl)
			.success(function(data) {
				cb(data);
			})
			.error(function(e) {
				cb(false);
			})		
	}

	return {
		getHotSpots   : getHotSpots,
		getDestaques  : getDestaques,
		getTopSongs   : getTopSongs,
		doSuggest     : doSuggest,
		getSong       : getSong,
		getLangVideos : getLangVideos
	}
}])

.factory('timeoutChain', ['$timeout', function($timeout){

	function Chain(context){
		context = context || {};
		var list = []
		, self = this
		, step = 0
		, substep = 0;

		function run(){
			if(list.length){
				$timeout(list[0][0], list[0][1]);
			}
		}

		this.noop = function(){
			list.push([function(){            
				list.shift();
				run();
			}, 0]);
			return self;
		}

		this.add = function(f, t){
			list.push([function(){
				substep = 0;
				if(f.call(context, step++, 0) !== false){
					list.shift();
					run();
				}
			}, t || 0]);
			return self;
		}

		this.loop = function(f, t){
			list.push([function(){
				if(!f.call(context, step, substep++)){
					substep = 0;
					list.shift();
				}
				run();
			}, t || 0]);
			return self;
		}

		this.run = function(){
			run();
		}
	}

	return function(context){
		return new Chain(context || this);
	}
}])

.factory('ValueAnimator', ['$timeout', function($timeout){

	function loop(data){
		data.callback(data.start + (data.delta * (data.progress / 100)), data.progress);
		if(data.progress < 100){
			data.progress += 10;
			$timeout(function(){
				loop(data);
			}, Math.round(data.progress));
		}
	}

	function animate(start, end, callback){
		loop({
			start:start,
			end:end,
			delta:end-start,
			progress:0,
			callback:callback
		});
	}

	function Animator(callback){
		this.callback = callback;
	}

	Animator.prototype = {
		loop:function(time){
			var self = this;
			this.progress = Math.max(0, Math.min((time - this.startTime) / this.time, 1));
			this.ratio = this.ease(this.progress);
			this.callback.call(this);
			if(this.progress < 1){
				requestAnimationFrame(function(){
					self.loop(Date.now());
				});
			}
		},
		run:function(time, ease){
			this.startTime = Date.now();
			this.endTime = this.startTime + time;
			this.time = time;
			this.ease = ease || function(x){return x;};
			this.loop(this.startTime);
		},
		getValue:function(start, end){
			return start + ((end - start) * this.ratio);
		},
		setCallback:function(callback){
			this.callback = callback;
			return this;
		}
	};

	return {
		animate:animate,
		create:function(){
			return new Animator();
		}
	}
}])

.factory('VideoModel', ['$http', function($http){

	function onGetFirstValidVideo(context, data, error){
		var videoID = context.videoList[context.index];
		if(!data){
			error = 'no data';
		} else if(data.error){
			error = data.error && data.error.message;
		}
		if(error){
			context.index++;
			context.errors.push(error);
			getFirstValidVideoAux(context);
		} else {
			context.callback(videoID, null);
		}
	}

	function getFirstValidVideoAux(context){
		if(context.index < context.videoList.length){
			var videoId = context.videoList[context.index];
			$http
				.get('https://www.googleapis.com/youtube/v3/videos?id='+videoId+'&part=contentDetails&videoEmbeddable=true&key=AIzaSyC5r8CVC6q8hcbB79qYH871xZ114F_uOY8')
				.success(function(data){
					onGetFirstValidVideo(context, data, null);
				})
				.error(function(e){
					onGetFirstValidVideo(context, false, e || 'request fail');
				});
		} else {
			context.errors.push('no valid video found');
			context.callback(false, 'no valid video found', context.errors);
		}
	}

	function getFirstValidVideoID(videoList, callback){
		getFirstValidVideoAux({
			index     :0,
			videoList :videoList,
			callback  :callback,
			errors    :[]
		});
	}

	function getImage(ytID, quality){
		quality = quality || 0;
		return 'http://img.youtube.com/vi/'+ytID+'/'+quality+'.jpg'
	}

	return {
		getFirstValidVideoID:getFirstValidVideoID,
		getImage:getImage
	}
}])

.factory('YoutubeVagalumePlayer', ['$rootScope', '$timeout', '$window', 'postMessenger', function($rootScope, $timeout, $window, postMessenger){

	var youtubeVagalumePlayerInstance = null
	,   iframe = document.getElementById('iframeYoutube')
	,   vagalumePlayer = null;

	// Events
	$window.addEventListener("message", function(event) {
		switch(event.data) {
			case 'playing':
				vagalumePlayer.currentState = 'playing';
				vagalumePlayer.timeLoop.play();
				$rootScope.$emit('vagalume.player.playing');
				break;
			case 'ended':
				vagalumePlayer.timeLoop.stop();
				vagalumePlayer.currentState = 'ended';
				$rootScope.$emit('vagalume.player.ended');
				break;
			case 'buffering':
				$rootScope.$emit('vagalume.player.buffering');
				break;
			case 'paused':
				vagalumePlayer.timeLoop.stop();
				vagalumePlayer.currentState = 'paused';
				$rootScope.$emit('vagalume.player.paused');
				break;
			case 'ready':
				$rootScope.$emit('vagalume.player.ready');
				break;
			case 'error':
				$rootScope.$emit('vagalume.player.error');
				break;
		}
	}, false);

	// Loop prototype
	var loopID = 1;

	function Loop(callback, interval, name){
		this.running = false;
		this.interval = interval
		this.callback = callback || null;
		this.args = [];
		this.id = name || loopID++;
		this.timeout = null;
	}

	Loop.loop = function(loopInstance){
		if(loopInstance.running){
			if(loopInstance.callback){
				loopInstance.callback.apply(loopInstance, loopInstance.args);
				loopInstance.timeout = $timeout(function(){
					Loop.loop(loopInstance);
				}, loopInstance.interval);
			}
		}
	}

	Loop.prototype = {
		play:function(){
			this.args = arguments;
			if(!this.running){
				var self = this;
				this.running = true;
				this.timeout = $timeout(function(){
					Loop.loop(self);
				}, this.interval);
			}
			return this;
		},
		stop:function(){
			if(this.running){
				this.running = false;
				if(this.timeout){
					$timeout.cancel(this.timeout);
				}
				this.args = [];
			}
		}
	}

	// YoutubeVagalumePlayer prototype
	function YoutubeVagalumePlayer(){
		var self = this;
		vagalumePlayer = this;

		this.timeLoop = {
			timeout: null,
			play: function() {
				this.timeout = $timeout(function() {
					self.timeLoop.loop();
				}, 100);
			},
			stop: function() {
				$timeout.cancel(this.timeout);
			},
			loop: function() {
				var self = this;
				postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'getPlayerState')
				.then(function(currentState) {
					if(currentState === 1) {
						postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'getCurrentTime')
						.then(function(currentTime) {
							$rootScope.$emit('vagalume.player.timescan', self, currentTime || 0);
							self.play();
						});
					}
				});
			}
		}

		this.volumeLoop = new Loop(function(type, left){
			if(left--){
				switch(type){
					case 'pause':
						postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'setVolume', 10 * left);
						break;
					case 'play':
						postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'setVolume', 10 * left);
						break;
				}
				this.args[1] = left;
			} else {
				switch(type){
					case 'pause':
						postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'pauseVideo');
						this.stop();
						break;
				}
			}
		}, 50, 'volume');
	}

	$rootScope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams){
		if(youtubeVagalumePlayerInstance) {
			youtubeVagalumePlayerInstance.destroy();
		}
	});

	YoutubeVagalumePlayer.prototype = {
		getDuration:function(){
			return this.player.getDuration();
		},
		pause:function(enableTransition){
			this.timeLoop.stop();
			if(enableTransition){
				this.currentState = 'pausing';
				this.volumeLoop.play('pause', 10);
				$rootScope.$emit('vagalume.player.pausing', this);
			} else {
				postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'pauseVideo');
			}
			return this;
		},
		play:function(enableTransition){
			this.timeLoop.play();
			postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'playVideo');
			this.currentState = 'playing';
			if(enableTransition){
				this.volumeLoop.play('play', 10);
			} else {
				this.volumeLoop.stop();
				postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'setVolume', 100);
			}
			return this;
		},
		destroy:function() {
			this.timeLoop.stop();			
		},
		reload:function(){
			this.currentState = 'none';
			return this;
		},
		seekTo:function(time){
			postMessenger.callFunction('#iframeYoutube', 'youtubePlayer', 'seekTo', time);
			return this;
		}
	};

	return {
		create:function(){
			return youtubeVagalumePlayerInstance || (youtubeVagalumePlayerInstance = (new YoutubeVagalumePlayer()).reload());
		}
	}
}])

.factory('postMessenger', function($q) {
	var objectMap = {}
	,   returnMap = {}
	,   returnID  = 0;

	// callFunction()
	var callFunction = function(windowObject, objectName, methodName, arg1, arg2, argN) {
		var args = Array.prototype.slice.call(arguments)
		,   win  = args.shift();

		if(typeof(win) == 'string') {
			win = document.querySelector(win);
			if(win) {
				win = win.contentWindow;
			}
		}

		if(win) {
			var id = returnID = ((returnID + 1) % 10000);
			win.postMessage({
				type: 'callFunction',
				args: args,
				id: id
			}, '*');

			return (returnMap[id] = $q.defer()).promise;
		}

		var defer = $q.defer();
		setTimeout(function() {
			defer.resolve(false, 'NO_WINDOW');
		}, 1);
		return defer.promise;
	}

	window.addEventListener('message', function(event) {
		if(event.data.type) {
			if(event.data.type == 'callFunction') {
				var args = event.data.args
				,   objectName = args.shift()
				,   methodName = args.shift()
				,   id         = event.data.id
				,   error      = null
				,   result     = false;

				if(objectMap[objectName]) {
					if(objectMap[objectName][methodName]) {
						result = objectMap[objectName][methodName].apply(objectMap[objectName], args)	
						if(typeof(result) == 'object') {
							result = false;
						}
					} else {
						error = 'NO_FUNCTION';
					}
				} else {
					error = 'NO_OBJECT';
				}

				event.source.postMessage({
					id: id,
					result: result || null,
					type: 'returnFunction',
					error: error
				}, '*');
			} else if (event.data.type == 'returnFunction') {
				var defer = returnMap[event.data.id];
				defer.resolve(event.data.result, event.data.error);
				delete returnMap[event.data.id];
			}
		}
	}, false);

	var registerObject = function(name, object)  {
		objectMap[name] = object;
	}

	return {
		callFunction: callFunction,
		registerObject: registerObject
	}
})

// Directives

.directive('btnPopOver',
	['$ionicPopover',
	function ($ionicPopover) {
		return {
			restrict: 'C',
			replace: true,
			template:'<button class="more-info button button-icon icon ion-android-more-vertical flat pull-right" ng-click=popover.show($event)></button>',
			link: function (scope,elem,attr) {
				var popOverHtml = '<ion-popover-view>'
				+ '<div class=list>'
				+  '<a ng-show=isGameVideo ng-click="openChangeSubtitile()" class="item item-icon-left">'
				+    '<i class="icon ion-arrow-swap"></i>'
				+    'Trocar legenda'
				+  '</a>'
				+  '<a ng-show=isGameVideo ng-click="openChangeVideo()" class="item item-icon-left">'
				+    '<i class="icon ion-ios-videocam"></i>'
				+    'Trocar Vídeo'
				+  '</a>'
				+  '<a ng-show="isGameVideo && enableAnimation" ng-click="toggleAnimation()" class="item item-icon-left">'
				+    '<i class="icon ion-flash-off"></i>'
				+    'Desativar animações'
				+  '</a>'
				+  '<a ng-show="isGameVideo && !enableAnimation" ng-click="toggleAnimation()" class="item item-icon-left">'
				+    '<i class="icon ion-flash"></i>'
				+    'Ativar animações'
				+  '</a>'
				+ '</div>'
				+ '</ion-popover-view>';
				scope.popover = $ionicPopover.fromTemplate(popOverHtml, {scope: scope})
			}
		};
	}
])

.directive('phraseOptions', ['$timeout', 'timeoutChain', '$rootScope',function($timeout, timeoutChain, $rootScope) {
	var template =
	'<div class=phrase-options>'
		+'<ul class=users-current>'
			+'<li class=col ng-style="option.doneStyle" ng-class="{ohide: option.hide, animated: option.animate}" ng-repeat="option in optionsDone"><span class=to ng-click="back($index, $event)">{{option.text}}</span></li>'
		+'</ul>'
		+'<ul class=suggest-words>'
			+'<li class="" ng-style="option.infoStyle" ng-class="{ohide: option.hide, animated: option.animate}" ng-repeat="option in optionsInfo"><span class=from ng-click="go($index, $event)">{{option.text}}</span></li>'
		+'</ul>'
		+'<div class=floatingButton ng-show="floatingButton.show" ng-style="floatingButton.style"><span ng-class="{to: floatingButton.to, from:!floatingButton.to}">{{floatingButton.text}}</span></div>'
	+'</div>';


	function link ($scope, element, attrs) {

		var floatingButton = document.querySelector('.floatingButton')
		,   phraseOptions  = document.querySelector('.phrase-options')
		,   lock           = false;

		$scope.floatingButton = {};

		$scope.$watch('optionsInfo', function(){
			$scope.floatingButton.show = false;
			var li = document.querySelectorAll('.phrase-options .users-current li');
			if(li.length) {
				var ul = li[0].parentNode;
				for (var i = 0; i < li.length; i++) {
					ul.removeChild(li[i]);
				};
			}
		});

		function move(from, to, index, li) {
			if(lock) return;
			$rootScope.$emit('phraseOptions.move');
			lock = true;
			var phraseOptionsBox  = phraseOptions.getBoundingClientRect()
			,   list1   = from == 'done' ? $scope.optionsDone : $scope.optionsInfo
			,   list2   = to == 'info' ? $scope.optionsInfo : $scope.optionsDone;

			li = li.getBoundingClientRect();

			$scope.floatingButton.text = list1[index].text;

			$scope.floatingButton.style = {
				top: li.top - phraseOptionsBox.top + 'px',
				left: li.left - phraseOptionsBox.left + 'px'
			};

			$scope.floatingButton.show = true;

			list1[index].hide = true;
			list2.push(list1[index]);

			list1[index].infoStyle = {};
			list1[index].doneStyle = {};

			timeoutChain({})
			.noop()
			.add(function(){
				list1[index].animate = false;
				var style = {
					'width': li.width + 'px'
				};
				if(from == 'info') {
					list1[index].infoStyle = style;
				} else {
					list1[index].doneStyle = style;
				}
			}, 10)
			.loop(function(step, subStep) {
				var toClass = to == 'info' ? '.suggest-words' : '.users-current';
				var box = document.querySelector('.phrase-options '+toClass+' li:last-child');
				this.box = box.getBoundingClientRect();
				return this.box.width == 0 && subStep < 60;
			}, 50)
			.add(function() {
				list1[index].animate = true;
			}, 10)
			.add(function() {
				var floatingButtonBox = floatingButton.getBoundingClientRect()
				,   left = this.box.left - floatingButtonBox.left
				,   top  = this.box.top - floatingButtonBox.top;

				$scope.floatingButton.to = to != 'info';
				$scope.floatingButton.style.transform = 'translate('+left+'px, '+top+'px)';
				$scope.floatingButton.style['-webkit-transform'] = 'translate('+left+'px, '+top+'px)';
				$scope.floatingButton.style['-ms-transform'] = 'translate('+left+'px, '+top+'px)';
				$scope.floatingButton.style['-o-transform'] = 'translate('+left+'px, '+top+'px)';
				$scope.floatingButton.style['-moz-transform'] = 'translate('+left+'px, '+top+'px)';
			}, 10)
			.add(function(){
				if(from == 'info') {
					list1[index].infoStyle = {
						'width': '0',
						'margin': '0px 0px 5px 0px'
					}
				} else {
					list1[index].doneStyle = {
						'width': '0',
						'margin': '0px 0px 5px 0px'
					}
				}
			}, 100)
			.add(function() {
				list1[index].hide = false;
				list1[index].infoStyle = {};
				list1[index].doneStyle = {};
				$scope.floatingButton.show = false;

				for(var i=index; i<list1.length - 1; i++){
					list1[i] = list1[i+1];
				}
				list1.pop();
			}, 100)
			.add(function(){
				lock = false;
			}, 10)
			.run();
		}

		$scope.back = function(index, $event) {
			move('done', 'info', index, $event.target);
		}

		$scope.go = function(index, $event) {
			move('info', 'done', index, $event.target);
		}
	}

	return {
		restrict: 'E',
		replace: true,
		scope: {
			optionsInfo: '=info',
			optionsDone: '=done'
		},
		template: template,
		link: link
	};
}])

.directive('lyricsScroller', ['$timeout', 'ValueAnimator', function($timeout, ValueAnimator){

	var template = '<ul><li ng-repeat="item in list track by $index" ng-class="{current: $index == index}">{{item}}</li></ul>';

	function link($scope, element, attrs){
		var mask = element[0]
		  , ul = mask.querySelector('ul')
		  , start = 0
		  , end = 0
		  , animator = ValueAnimator.create().setCallback(function(){
			mask.scrollTop = this.getValue(start, end);
		  });

		$scope.$watch('index', function(){
			$timeout(function(){
				start = mask.scrollTop;
				end = getScrollPosition(mask, ul);
				animator.run(200);
			}, 100);
		});
	}

	function getScrollPosition(mask, ul){
		var li = ul.querySelector('li.current');
		if(!li) return 0;
		var maskBox = mask.getBoundingClientRect()
		  , ulBox   = ul.getBoundingClientRect()
		  , liBox   = li.getBoundingClientRect()
		  , max     = ulBox.height - maskBox.height
		  , top     = (liBox.top - ulBox.top) - (maskBox.height / 2) + (liBox.height / 2);
		return top;
	}

	return {
		restrict:'C',
		scope:{list: '=list', index: '=index'},
		template: template,
		link:link
	};
}])

.directive('listRank', function() {

	var genHtml = function(elem, attr) {
		var html = '';
		switch (attr.type) {

			case 'week':
				html = '<a data-iteration="{{key + 1}}" ng-repeat="(key, user) in weekRank" class="item item-avatar item-icon-right waves-effect waves-button waves-dark" >'
				+           '<img ng-show="user.picID" src="http://s2.vagalume.com/dynimage_user/{{user.picID}}.jpg">'
				+           '<img ng-hide="user.picID" src="http://s2.vagalume.com/images/my/no_photo_boy.jpg">'
				+           '<h2>{{user.name}}</h2>'
				+           '<p>{{user.score | number:0}} pontos</p>'
				+      '</a>';
				break;

			case 'month':
				html = '<a data-iteration="{{key + 1}}" ng-repeat="(key, user) in monthRank" class="item item-avatar item-icon-right waves-effect waves-button waves-dark" >'
				+           '<img ng-show="user.picID" src="http://s2.vagalume.com/dynimage_user/{{user.picID}}.jpg">'
				+           '<img ng-hide="user.picID" src="http://s2.vagalume.com/images/my/no_photo_boy.jpg">'
				+           '<h2>{{user.name}}</h2>'
				+           '<p>{{user.score | number:0}} pontos</p>'
				+      '</a>';
				break;

			case 'general':
				html = '<a data-iteration="{{key + 1}}" ng-repeat="(key, user) in generalRank" class="item item-avatar item-icon-right waves-effect waves-button waves-dark" >'
				+           '<img ng-show="user.picID" src="http://s2.vagalume.com/dynimage_user/{{user.picID}}.jpg">'
				+           '<img ng-hide="user.picID" src="http://s2.vagalume.com/images/my/no_photo_boy.jpg">'
				+           '<h2>{{user.name}}</h2>'
				+           '<p>{{user.score | number:0}} pontos</p>'
				+      '</a>';
				break;
		}

		return html;
	}

	return {
		restrict: 'A',
		template: genHtml,
	}
})

.directive('focusMe', ['$timeout', function($timeout) {
	return {
		scope: { trigger: '=focusMe' },
		link: function(scope, element) {
			scope.$watch('trigger', function(value) {
				if(value === true) {
					$timeout(function() {
						element[0].focus();
						scope.trigger = false;
					});
				}
			});
		}
	};
}])

// Controllers

.controller('MainController', ['$scope', '$cordovaToast', '$rootScope', '$ionicPopup', '$timeout','$state', '$stateParams', 'GameVideo', 'GameModel', 'GamePhrase', 'sessionUser', '$ionicAnalytics', function($scope, $cordovaToast, $rootScope, $ionicPopup, $timeout, $state, $stateParams, GameVideo, GameModel, GamePhrase, sessionUser, $ionicAnalytics) {
	$scope.user = {};
	var eventMap = {}
	,   userLogged = $rootScope.$on('userLogged', function() {
		$scope.user = sessionUser.load();	
	});

	$scope.$on('$destroy', function() {
		for(var name in eventMap) {
			eventMap[name]();
		}
	})

	window.addEventListener('native.keyboardshow', function(){
		var div = document.querySelector('.home-content');
		if (div) {
			div.classList.add('keyboard');			
		}
	});

	window.addEventListener('native.keyboardhide', function(){
		var div = document.querySelector('.home-content');
		if (div) {
			div.classList.remove('keyboard');			
		}
	});

	$scope.goSettings = function() {
		window.open('https://auth-mobile.vagalume.com.br/settings/', '_system');
	}

	$scope.isMenuOpened = false;

	document.addEventListener("deviceready", onDeviceReady, false);

	function onDeviceReady() {
		window.addEventListener('native.keyboardhide', function() {
			if ($rootScope.search) {
				$rootScope.search = false;
				document.getElementById('suggest').blur();
			}
		});
		document.addEventListener("pause", onPause, false);
	}

	function onPause() {
		switch($state.current.name) {
			case 'video':
				if (!$rootScope.gameOver) {
					$scope.menuPopup('video');
				}
				break;
			case 'verse':
				if (!$rootScope.gameOver) {
					$scope.menuPopup('verse');
				}
				break;
		}
	}

	$scope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams){
		$scope.isMenuOpened = false;
	});

	$scope.reload = function() {
		location.reload();
	}

	$scope.logout = function() {
		sessionUser.logoff(function(){
			location.reload();
		});
	}

	$scope.toggleMenu = function() {
		$scope.isMenuOpened = $scope.isMenuOpened ? false : true;
	}

	$rootScope.sharePopup = function(score, song, band, level) {
		var message = '';
		if (score > 0) {
			message = 'Consegui fazer '+score+' pts no nível '+level+' do VAGALUME Aprenda nesta música, e vc? Jogue agora!';
		} else {
			message = 'Quantos pontos você consegue fazer no nível '+level+' do VAGALUME Aprenda nesta música? Jogue agora!';
		}
		window.plugins.socialsharing.share(message, null, null, 'http://aprenda.vagalume.com.br/'+$stateParams.artUrl+'/'+$stateParams.musUrl+'.html')
	}

	// Função para comparar arrays
	Array.prototype.equals = function (array) {
		if(array instanceof Array){
			if (this.length != array.length)
				return false;

			for (var i = this.length; i--;) {
				if (this[i] instanceof Array && array[i] instanceof Array) {
					if (!this[i].equals(array[i]))
						return false;
				} else if (this[i] != array[i]) {
					return false;
				}
			}
			return true;
		}
		return false;
	};

	// Função para pegar o número da semana atual
	Date.prototype.getWeek = function() {
		var onejan = new Date(this.getFullYear(),0,1);
		return Math.ceil((((this - onejan) / 86400000) + onejan.getDay()+1)/7);
	}

	// Popup "Pause do jogo"
	$scope.menuPopup = function(type) {
		$scope.type = type;

		// Sub popup para sair do jogo
		$scope.quitGame = function() {
			var confirmPopup = $ionicPopup.confirm({
				title: 'Sair do Jogo',
				template: 'Você deseja realmente sair? Seus pontos não serão computados!',
				cancelText: 'Não',
				okText: 'Sim'
			});

			confirmPopup.then(function(res) {
				if(res) {
					$ionicAnalytics.track('Sair do jogo', {
						'modo': type,
						'nível': $stateParams.level
					});
					$scope.$apply();
					$state.go('aprenda.home');
				} else {
					returnGame();
				}
			});
		};

		// Sub popup para troca de dificuldade
		$scope.changeLevel = function() {
			var confirmPopup = $ionicPopup.confirm({
				title: 'Sair do Jogo',
				template: 'Você deseja realmente sair? Seus pontos não serão computados!',
				cancelText: 'Não',
				okText: 'Sim'
			});

			confirmPopup.then(function(res) {
				if(res) {
					// Cada modo de jogo redirecionará para uma url específica
					$ionicAnalytics.track('Trocar dificuldade', {
						'modo': type,
						'nível': $stateParams.level
					});
					switch(type) {
						case 'video':
						case 'texto':
							$state.go('game-mode', {artUrl: $stateParams.artUrl, musUrl: $stateParams.musUrl});
						break;
					}
				} else {
					returnGame();
				}
			});
		};

		// Sub popup para reiniciar o jogo
		$scope.restart = function() {
			var confirmPopup = $ionicPopup.confirm({
				title: 'Sair do Jogo',
				template: 'Você deseja realmente reiniciar o jogo? Seus pontos não serão computados!',
				cancelText: 'Não',
				okText: 'Sim'
			});

			confirmPopup.then(function(res) {
				if(res) {
					$ionicAnalytics.track('Reiniciar jogo', {
						'modo': type,
						'nível': $stateParams.level
					});
					myPopup.close();
					location.reload();
				} else {
					returnGame();
				}
			});
		};

		function returnGame() {
			switch($scope.type) {
				case 'verse':
					if(window.Connection) {
						if(navigator.connection.type == Connection.NONE) {
							$cordovaToast.showShortBottom('Verifique a conexão com a internet').then(
								function (success) {},
								function (error) {}
							);
							e.preventDefault();								
						} else {
							GamePhrase.get().unpause();										
						}
					}
					break;
				case 'video':
					if(window.Connection) {
						if(navigator.connection.type == Connection.NONE) {
							$cordovaToast.showShortBottom('Verifique a conexão com a internet').then(
								function (success) {},
								function (error) {}
							);
							e.preventDefault();								
						} else {
							if(GameVideo.get().answerWait) {
								GameVideo.get().unpause();
								GameVideo.get().repeat();
							} else {
								GameVideo.get().unpause();											
							}
						}
					}
					break;
			}
		}

		// Popup principal do "pause"
		var myPopup = $ionicPopup.show({
			template: '',
			cssClass: 'menu-popup',
			title: 'Jogo Pausado',
			scope: $scope,
			buttons: [
				{
					text: 'Continuar',
					type: 'button-positive',
					onTap: function(e) {
						returnGame();
					}
				},
				{
					text: 'Reiniciar',
					onTap: function(e) {
						$timeout(function() {
							$scope.restart();
						});
					}
				},
				{
					text: 'Trocar Dificuldade',
					onTap: function(e) {
						$timeout(function() {
							$scope.changeLevel();
						}, 20);
					}
				},
				{
					text: 'Sair',
					onTap: function(e) {
						$timeout(function() {
							$scope.quitGame();
						}, 20);
					}
				}
			]
		});

		switch(type) {
			case 'verse':
				GamePhrase.get().pause();
				break;
			break;
			case 'video':
				GameVideo.get().pause();
			break;
		}
	};

	// Popup para sair do jogo
	$scope.quitGame = function(type) {
		var confirmPopup = $ionicPopup.confirm({
			title: 'Sair do Jogo',
			template: 'Você deseja realmente sair? Seus pontos não serão computados!',
			cancelText: 'Não',
			okText: 'Sim'
		});

		confirmPopup.then(function(res) {
			if(res) {
				$ionicAnalytics.track('Sair do jogo', {
					'modo': type,
					'nível': $stateParams.level
				});
				$state.go('aprenda.home');
			}
		});
	};
}])

.controller('HomeController',[
	'$scope', '$timeout', 'RankModel', '$ionicSlideBoxDelegate', '$ionicScrollDelegate', '$ionicLoading', '$rootScope', '$cordovaToast', 'RequestModel', '$ionicAnalytics', '$ionicModal', '$state',
	function($scope, $timeout, RankModel, $ionicSlideBoxDelegate, $ionicScrollDelegate, $ionicLoading, $rootScope, $cordovaToast, RequestModel, $ionicAnalytics, $ionicModal, $state) {

	var loading = function() {
		$ionicLoading.show({template: '<ion-spinner icon=ios class=spinner-calm></ion-spinner>', noBackdrop: true});
	}
	,   finish = function() {
		$ionicLoading.hide();
	}
	,   resultHotspot = function(data){
		var hotArray = [{
			'pic_src': 'img/banner-idiomas.jpg',
			'musTitle': '',
			'title': '',
			'link': ''
		},
		{
			'pic_src': 'img/bannerRandom.jpg',
			'musTitle': '',
			'title': '',
			'link': ''
		}];
		for (var i = 0,ii = data.length; i < ii; i++) {
			hotArray.push(data[i]);
		};
		$scope.HotSpotSlide.slides = hotArray;
		// Atualiza o slidebox do Ionic
		$ionicSlideBoxDelegate.update();
		// Carrega os dois primeiros slides
		if($scope.HotSpotSlide.slides.length >= 2) {
			$scope.HotSpotSlide.slides[0].style = {
				'background-image': 'url('+$scope.HotSpotSlide.slides[0].pic_src+')'
			};
			$scope.HotSpotSlide.slides[1].style = {
				'background-image': 'url('+$scope.HotSpotSlide.slides[1].pic_src+')'
			};			
		}
		// Adiciona a classe para mostrar o slider do Hot Spot
		$scope.HotSpotSlide.slideClass = 'show';
	}
	,   resultTopSongs = function(data){
		try {
			$scope.topSongs = data;
		} catch (e) {
		}
		finish();
	}
	,   resultMostPlayed = function(data){
		$scope.mostPlayed = data;
		finish();
	}
	,   getTop = function(action, force) {
		$scope.activeTab = action;
		switch(action) {
			case 'songs':
				if (!$scope.topSongs) loading();
				if (!$scope.topSongs || force) RankModel.getTopSongs(resultTopSongs);
				break;
			case 'mostPlayed':
				if (!$scope.mostPlayed) loading();
				if (!$scope.mostPlayed || force) RankModel.getMostPlayed(resultMostPlayed);
				break;
		}
	}

	$scope.clickBanner = function(code) {
		switch(code) {
			case 0:
				$scope.openLangModal();
				break;
			case 1:
				$state.go('random-mode');
				break;
		}
	}

	$scope.openLangModal = function() {		
		$ionicModal.fromTemplateUrl('lang-choose.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$ionicAnalytics.track('Escolha de Idiomas');
			$scope.langModal = modal;
			$scope.langModal.show();
		});	
	}

	$scope.closeLangModal = function() {
		$scope.langModal.remove();
	}

	$scope.slideHasChanged = function($index) {
		if(($index + 1) != $scope.HotSpotSlide.slides.length) {
			var nextSlide = $scope.HotSpotSlide.slides[$index+1];
			if(!nextSlide.style) {
				nextSlide.style = {
					'background-image': 'url('+nextSlide.pic_src+')'
				};
			}			
		}
	}

	$scope.toggleSuggestBox = function() {
		$rootScope.search = $rootScope.search ? false : true;
		if ($rootScope.search) {
			$ionicAnalytics.track('Suggest');
		}
	}

	// Suggest de pesquisa de músicas do menu
	$scope.suggest = function() {
		if(window.Connection) {
			if(navigator.connection.type != Connection.NONE) {
				RequestModel
				.doSuggest($scope.suggestData.search, function(data){
					$rootScope.search = true;
					var docs = data.response.docs;
					for (var i = 0; i < docs.length; i++) {
						docs[i].artUrl  = docs[i].url.match(/^\/([^\/]+)\/([^\/.]+)/)[1];
						docs[i].musUrl  = docs[i].url.match(/^\/([^\/]+)\/([^\/.]+)/)[2];
						docs[i].musName = docs[i].title;
						docs[i].musBand = docs[i].band;
					};
					$scope.suggestData.songs = docs;
				})
			} else {
				$cordovaToast.showShortCenter('Verifique a conexão com a internet').then(
					function (success) { },
					function (error) { }
				);
			}
		}
	}

	// Escopo de configuração do slide do hotspot
	$scope.HotSpotSlide = {
		index: 0,
		hide: false
	};
	$scope.marginRoll = false;
	$scope.searchTab = false;

	// Pega o hotspot
	RankModel.getHotSpots(resultHotspot)

	$scope.clickTab = getTop;
	$scope.topSongs = $scope.mostPlayed = $scope.hotspot = false;
	$scope.activeTab = '';

	getTop('mostPlayed');
}])

.controller('TabController', ['$scope', function($scope) {
	$scope.tabsControl = {
			index: 1
		};

	// Controle das abas do menu de Ranking e Tops
	$scope.selectTab = function(setTab) {
		$scope.tabsControl.index = setTab;
	}
	$scope.isSelected = function(checkTab) {
		return $scope.tabsControl.index == checkTab;
	}
}])

.controller('MenuController',[
	'$scope', '$timeout', 'RequestModel', '$rootScope', '$cordovaToast',
	function($scope, $timeout, RequestModel, $rootScope, $cordovaToast) {
	$rootScope.search = false;
	$scope.suggestData  = { "songs" : [], "search" : '' };

	$scope.enableSearch = function() {
		$rootScope.search = !$rootScope.search ? true : false;		
	}

	// Suggest de pesquisa de músicas do menu
	$scope.suggest = function() {
		if(window.Connection) {
			if(navigator.connection.type != Connection.NONE) {
				RequestModel
				.doSuggest($scope.suggestData.search, function(data){
					for (var i = 0; i < data.response.docs.length; i++) {
						data.response.docs[i].artUrl  = data.response.docs[i].url.match(/^\/([^\/]+)\/([^\/.]+)/)[1];
						data.response.docs[i].musUrl  = data.response.docs[i].url.match(/^\/([^\/]+)\/([^\/.]+)/)[2];
						data.response.docs[i].musName = data.response.docs[i].title;
						data.response.docs[i].musBand = data.response.docs[i].band;
					};
					$scope.suggestData.songs = data.response.docs;
				})
			} else {
				$cordovaToast.showShortCenter('Verifique a conexão com a internet').then(function(success) {
				}, function (error) {
				});
			}
		}
	}	
}])

.controller('RandomModeController',[
	'$scope', '$state', '$stateParams', 'GameModel', '$ionicLoading',
	function($scope, $state, $stateParams, GameModel, $ionicLoading) {
	$scope.templates =
	[{
		name: 'game-lang',
		url: 'app/Game/Random/mode.html'
	},
	{
		name: 'game-level',
		url: 'app/Game/Random/level.html'
	}];

	if (window.Connection) {
		if(navigator.connection.type == Connection.NONE) {
			$state.go('aprenda.no-connection');
		}
	}

	$scope.template = $scope.templates[0];

	$scope.tabsControl = {
		index: 1
	};

	$scope.randomChoose = {
		mode: '',
		level: ''
	}

	$scope.change = function() {
		$scope.template = $scope.templates[1];
	}

	$scope.levelClick = function(level) {
		$scope.randomChoose.level = level;
		// TODO: Chamar loading
		$ionicLoading.show({template: '<ion-spinner icon="android" class="spinner spinner-android"></ion-spinner>', noBackdrop: true});
		// Fazer a requisição para pegar música aleatórias
		GameModel.getRandomSongs($scope.randomChoose.mode, function(data) {
			$ionicLoading.hide();
			if ($scope.randomChoose.mode == 'texto') {
				$scope.randomChoose.mode = 'verse';
			}
			$state.go($scope.randomChoose.mode, {
				artUrl: data[0].artSlug,
				musUrl: data[0].musSlug,
				level: $scope.randomChoose.level
			});
		});
	}

	// Controle das abas do menu de Ranking e Tops
	$scope.selectTab = function(setTab) {
		$scope.tabsControl.index = setTab;
		switch(setTab) {
			case 1:
				$scope.randomChoose.mode = 'video';
				break;
			case 2:
				$scope.randomChoose.mode = 'texto';
				break;
		}
		$scope.template = $scope.templates[1];
	}
	$scope.isSelected = function(checkTab) {
		return $scope.tabsControl.index == checkTab;
	}
}])

.controller('PlayModeController',[
	'$scope', '$state', '$timeout', 'ArtistModel', '$stateParams', '$ionicLoading', '$cordovaDevice',
	function($scope, $state, $timeout, ArtistModel, $stateParams, $ionicLoading, $cordovaDevice) {

	$scope.templates =
	[{
		name: 'game-mode',
		url: 'app/Game/mode.html'
	},
	{
		name: 'game-level',
		url: 'app/Game/level.html'
	}];

	$scope.template = $scope.templates[0];
	$scope.tabsControl = {
		index: 1
	};

	$scope.oldVersion = false;
	var deviceType = (navigator.userAgent.match(/iPad/i))  == "iPad" ? "iPad" : (navigator.userAgent.match(/iPhone/i))  == "iPhone" ? "iPhone" : (navigator.userAgent.match(/Android/i)) == "Android" ? "Android" : (navigator.userAgent.match(/BlackBerry/i)) == "BlackBerry" ? "BlackBerry" : "null";
	if(deviceType == 'iPhone') {
		var version = parseFloat($cordovaDevice.getVersion());
		if(version < 8.0) {
			$scope.oldVersion = true;
		}
	}

	if (window.Connection) {
		if(navigator.connection.type == Connection.NONE) {
			$state.go('aprenda.no-connection');
		}
	}

	$scope.change = function() {
		$scope.template = $scope.templates[1];
	}

	$scope.finished = false;

	var loading = function() {
		$ionicLoading.show({template: '<ion-spinner icon="android" class="spinner spinner-android"></ion-spinner>', noBackdrop: true});
	}
	,   finish = function() {
		$ionicLoading.hide();
	}

	// Controle das abas do menu de Ranking e Tops
	$scope.selectTab = function(setTab) {
		$scope.tabsControl.index = setTab;
		$scope.template = $scope.templates[1];
	}
	$scope.isSelected = function(checkTab) {
		return $scope.tabsControl.index == checkTab;
	}

	$scope.song = {};

	// Controller da tela para escolher um tipo de jogo de uma música,
	// caso não encontre redireciona para a Home do Aprenda
	loading();
	ArtistModel
		.getSong($stateParams.artUrl, $stateParams.musUrl, function(song){
			if(song && song.type != 'song_notfound' && song.type != 'notfound') {
				$scope.song = song;
				finish();
				$scope.finished = true;
			} else {
				// Não encontrou a música
				finish();
				$state.go('aprenda.home');
			}
		})
}])

.controller('GameVideoController',[
	'$scope', '$cordovaToast', '$ionicHistory', '$state', '$timeout', '$cordovaVibration', 'ArtistModel', '$stateParams', '$http', '$ionicModal', 'GameModel', '$ionicPopup', 'GameVideo', '$ionicLoading', '$rootScope', 'RankModel', '$ionicGesture', '$document', 'YoutubeVagalumePlayer', 'postMessenger', '$sce', 'VideoModel', '$ionicAnalytics',
	function($scope, $cordovaToast, $ionicHistory, $state, $timeout, $cordovaVibration, ArtistModel, $stateParams, $http, $ionicModal, GameModel, $ionicPopup, GameVideo, $ionicLoading, $rootScope, RankModel, $ionicGesture, $document, YoutubeVagalumePlayer, postMessenger, $sce, VideoModel, $ionicAnalytics){
	
	$scope.game = null;
	$rootScope.gameOver = false;
	$scope.enableAnimation = true;
	var game     = $scope.game = GameVideo.get($stateParams.artUrl, $stateParams.musUrl)
	,   eventMap = {};

	// Verifica a conexão com a internet
	if (window.Connection) {
		if(navigator.connection.type == Connection.NONE) {
			$state.go('aprenda.no-connection');
		}
	}

	$scope.getVideoImage = function(videoID) {
		return VideoModel.getImage(videoID, 1);
	}

	$ionicModal.fromTemplateUrl('changeSubtitle.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function(modal) {
		$scope.subtitleModal = modal;
	});
	$scope.openChangeSubtitile = function() {
		GameModel.getSubtitlesOptions($scope.game.songId, $scope.game.songLangId, $scope.game.duration, function(data) {
			if(data.subtitles[0]) {
				$scope.subtitleOptions = data.subtitles;
			}
			$scope.popover.hide();
			$scope.game.pause();
			$scope.subtitleModal.show();
		});
	};
	$scope.closeChangeSubtitile = function() {
		$scope.game.unpause();
		$scope.subtitleModal.hide();
	};
	$scope.changeGameSubtitle = function(sID, time) {
		var confirmChangeSubtitle = $ionicPopup.confirm({
			title: 'Sair do Jogo',
			template: 'Tem certeza que deseja mudar a legenda? O jogo iniciará do começo e todos os pontos da partida serão perdidos.',
			buttons: [
			{ 
				text: 'Não',
				onTap: function(e) {
					confirmChangeSubtitle.close();
					return false;
				}
			},
			{
				text: 'Sim',
				type: 'button-positive',
				onTap: function(e) {
					$ionicAnalytics.track('Trocar legenda', {
						pointerID: $scope.game.songId,
						sID: $scope.game.sID
					});
					return true;
    			}
			}]
		});
		confirmChangeSubtitle.then(function(res) {
			if(res) {
				GameModel.denunciaSubtitle($scope.game.songId, $scope.game.sID, function(data) {
					$scope.subtitleModal.hide();
					$rootScope.sID = sID;
					$state.reload();
				})
			}
		});
	}

	$ionicModal.fromTemplateUrl('changeVideo.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function(modal) {
		$scope.videoModal = modal;
	});
	$scope.openChangeVideo = function() {
		$scope.popover.hide();
		$scope.game.pause();
		$scope.videoModal.show();
	};
	$scope.closeChangeVideo = function() {
		$scope.game.unpause();
		$scope.videoModal.hide();
	};
	$scope.changeVideo = function(videoID) {
		var confirmChangeVideo = $ionicPopup.confirm({
			title: 'Sair do Jogo',
			template: 'Tem certeza que deseja mudar o vídeo? O jogo iniciará do começo e todos os pontos da partida serão perdidos.',
			buttons: [
			{ 
				text: 'Não',
				onTap: function(e) {
					confirmChangeVideo.close();
					return false;
				}
			},
			{
				text: 'Sim',
				type: 'button-positive',
				onTap: function(e) {
					$ionicAnalytics.track('Trocar vídeo', {
						pointerID: $scope.game.songId
					});
					return true;
				}
			}]
		});
		confirmChangeVideo.then(function(res) {
			if(res) {
				$scope.videoModal.hide();
				$rootScope.videoId = videoID;
				$state.reload();
			}
		});
	}

	$scope.toggleAnimation = function() {
		if ($scope.enableAnimation) {
			$ionicAnalytics.track('Desativar animações');			
		}
		$scope.enableAnimation = $scope.enableAnimation ? false : true;
		$scope.popover.hide();
	}

	$scope.callFullScreen = function(){
		$scope.popover.hide();
		if ( document.fullscreenEnabled || document.webkitFullscreenEnabled
		||   document.mozFullScreenEnabled || document.msFullscreenEnabled
		) {
			if ( document.fullscreenElement || document.webkitFullscreenElement
			||   document.mozFullScreenElement || document.msFullscreenElement
			) {
				$scope.isFullScreen = false;
				var i = document;
				var callfs = i.exitFullscreen || i.webkitExitFullscreen || i.mozCancelFullScreen || i.msExitFullscreen;
			} else{
				$scope.isFullScreen = true;
				var i = document.querySelector('.pane');
				var callfs = i.requestFullscreen || i.webkitRequestFullscreen || i.mozRequestFullScreen || i.msRequestFullscreen;
			}
			if (callfs) callfs.apply(i);
		}
	}

	$scope.hideMenuBar = false;

	$scope.hideMenu = function() {
		$scope.popover.hide();
		$scope.hideMenuBar = true;
	}

	var level;

	$scope.song            = {};
	$scope.shake           = false;
	$scope.currentVerse    = {};
	$scope.isGameVideo     = true;
	$scope.loading         = {
		finished: false,
		gameStarted: false,
		message:  ''
	};
	$scope.shakeWord       = ' ';
	$scope.currentText     = '...';
	$scope.spinVideoLoader = false;

	// Variáveis do YouTube
	$scope.playerVars = {
		egm            : 0,
		iv_load_policy : 3,
		showinfo       : 0,
		real           : 0,
		enablejsapi    : 1,
		autoplay       : 0,
		wmode          : "transparent",
		controls       : 0		
	};

	// Configurações iniciais do jogo
	$scope.settings = {
		score: 0,
		words: 0,
		progress: {
			part: 0,
			total: 0
		}
	};

	$scope.ready = true;
	var finish = function(data) {
		if(data) {
			$timeout(function() {
				$scope.ready = true;
				$scope.loading.finished = true;
				$scope.loading.message = 'Toque no botão para iniciar o jogo!';
			}, 500);
		} else {
			$scope.ready = false;
			$scope.gameError = true;
			$scope.loading.message = 'Música sem legenda.';
			$scope.$apply();
		}
	},
	scrollSubtitle = function() {
		var top = game.currentVerseIndex * 30;
		if(game.currentVerseIndex > 0) {
			$scope.subtitleScroll = '-' + top + 'px';
		}
	}

	eventMap.computeAnswer = $rootScope.$on('computeAnswer', function(e, text, answerType, hasEnded) {
		var score = 0;
		switch(answerType) {
			case 'hit':
				var land       = document.querySelector('.land-bottom').getBoundingClientRect()
				,   blank      = document.querySelector('.subtitle-control b')
				,   blankPos   = blank.getBoundingClientRect()
				,   correct    = document.querySelector('button[data-text="'+text+'"]').getBoundingClientRect()
				,   transform  = 'translate(' + (correct.left - land.left) + 'px, ' + (correct.top - land.top) + 'px)'
				,   guess1     = {
					'-moz-transform': transform,
					'-ms-transform': transform,
					'-o-transform': transform,
					'-webkit-transform': transform,
					'transform': transform,
					opacity : 1
				};

				if ($scope.enableAnimation) {
					$scope.guess = guess1;
					$scope.textGuess = text;
					$scope.animateGuess = true;

					if (hasEnded) {
						$scope.animateWords = true;
					}					

					$timeout(function() {
						var translate  = 'translate(' + (blankPos.left - land.left) + 'px, ' + (blankPos.top - land.top) + 'px)'
						,   guess2     = {
							'-moz-transform': translate,
							'-ms-transform': translate,
							'-o-transform': translate,
							'-webkit-transform': translate,
							'transform': translate,
							'opacity'  : 0
						};

						if(hasEnded) {
							$timeout(function() {
								if(window.innerHeight > window.innerWidth){
									$scope.wordsStyle = {
										'-moz-transform': 'translateX(-200%)',
										'-ms-transform': 'translateX(-200%)',
										'-o-transform': 'translateX(-200%)',
										'-webkit-transform': 'translateX(-200%)',
										'transform': 'translateX(-200%)'
									}
								} else {
									$scope.wordsStyle = {
										'-moz-transform': 'translateX(100%)',
										'-ms-transform': 'translateX(100%)',
										'-o-transform': 'translateX(100%)',
										'-webkit-transform': 'translateX(100%)',
										'transform': 'translateX(100%)'
									}
								}
							}, 200);
						}

						if($scope.enableAnimation) {
							$scope.guess = guess2;						
						}

						$timeout(function() {
							$scope.animateGuess = false;
							updateSubtitle.apply(this, updateSubtitle.args);
							if(hasEnded) {
								game.replay();
							}
						}, hasEnded ? 500 : 200);
					}, 50);
				} else {
					if(hasEnded) {
						if(window.innerHeight > window.innerWidth){
							$scope.wordsStyle = {
								'-moz-transform': 'translateX(-200%)',
								'-ms-transform': 'translateX(-200%)',
								'-o-transform': 'translateX(-200%)',
								'-webkit-transform': 'translateX(-200%)',
								'transform': 'translateX(-200%)'
							}
						} else {
							$scope.wordsStyle = {
								'-moz-transform': 'translateX(100%)',
								'-ms-transform': 'translateX(100%)',
								'-o-transform': 'translateX(100%)',
								'-webkit-transform': 'translateX(100%)',
								'transform': 'translateX(100%)'
							}
						}						
					}

					updateSubtitle.apply(this, updateSubtitle.args);
					if(hasEnded) {
						game.replay();
					}
				}


				break;
			case 'skip':
				if (hasEnded) {
					$scope.animateWords = true;
				}
				$timeout(function() {
					if(hasEnded) {
						$timeout(function() {
							if(window.innerHeight > window.innerWidth){
								$scope.wordsStyle = {
									'-moz-transform': 'translateX(-200%)',
									'-ms-transform': 'translateX(-200%)',
									'-o-transform': 'translateX(-200%)',
									'-webkit-transform': 'translateX(-200%)',
									'transform': 'translateX(-200%)'
								}
							} else {
								$scope.wordsStyle = {
									'-moz-transform': 'translateX(100%)',
									'-ms-transform': 'translateX(100%)',
									'-o-transform': 'translateX(100%)',
									'-webkit-transform': 'translateX(100%)',
									'transform': 'translateX(100%)'
								}
							}
						}, 400);
					}

					$timeout(function() {
						updateSubtitle.apply(this, updateSubtitle.args);
						if(hasEnded) {
							game.replay();
						}
					}, hasEnded ? 800 : 50);
				}, 50);
				break;
		}
	})
	eventMap.endGame = $rootScope.$on('endGame', function(e, instance, reason) {
		switch(reason) {
			case 'timeEnd':
				$timeout(function() {
					openGameOverModal();
				}, 200);
				break;
			case 'blankEnd':
				// TODO: Mostrar mensagem para o usuário terminar o jogo
				break;
		}
	})
	eventMap.vagalumeBuffering = $rootScope.$on('vagalume.player.buffering', function() {
		$scope.startGame = true;
		$scope.$apply();
	})
	eventMap.endVideo = $rootScope.$on('endVideo', openGameOverModal)
	eventMap.subtitleChangeOptions = $rootScope.$on('subtitleChangeOptions', function(e, instance, isFirst) {
		if($scope.currentBlanks && $scope.currentBlanks.length){
			updateSubtitle.args = arguments;
		} else {
			updateSubtitle(e, instance, isFirst);
		}
	});

	function updateSubtitle(e, instance, isFirst){
		$timeout(function() {
			$scope.current = game.progressStep;
			var list = game.getAnswerOptions();
			$scope.currentBlanks = list;

			if(list.length && isFirst) {
				$scope.firstB = angular.element(document.querySelector('.subtitle-control span b'));
				if($scope.firstB) $scope.firstB.addClass('current-blank');

				$scope.firstS = angular.element(document.querySelector('.subtitle-control .bg-fade b'));
				if ($scope.firstS) $scope.firstS.addClass('current-shade');

				$scope.animateWords = false;
				$timeout(function() {
					$scope.wordsStyle = {
						'-moz-transform': 'translateX(100%)',
						'-ms-transform': 'translateX(100%)',
						'-o-transform': 'translateX(100%)',
						'-webkit-transform': 'translateX(100%)',
						'transform': 'translateX(100%)'
					}
					$timeout(function() {
						if($scope.enableAnimation) $scope.animateWords = true;
						$timeout(function() {
							$scope.wordsStyle = {
								'-moz-transform': 'translateX(0%)',
								'-ms-transform': 'translateX(0%)',
								'-o-transform': 'translateX(0%)',
								'-webkit-transform': 'translateX(0%)',
								'transform': 'translateX(0%)'
							}
						}, 10);
					}, 10);
				}, 10);
			}

			scrollSubtitle();
			$scope.settings.score = game.score.score;
			$scope.settings.words = game.score.chancesLeft;
			$scope.currentText = game.answerVerseText;
			$scope.nextVerseText = game.nextVerseText;
		}, 0);
	}

	eventMap.subtitleChange = $rootScope.$on('subtitleChange', function(e, instance) {
		$timeout(function() {
			scrollSubtitle();
			$scope.settings.score = game.score.score;
			$scope.settings.words = game.score.chancesLeft;
			$scope.currentText = game.answerVerseText;
			$scope.nextVerseText = game.nextVerseText;
		}, 0);
	})

	// Verificação de level da URL
	switch($stateParams.level) {
		case 'iniciante':
			level = 'beginner';
		break;
		case 'intermediario':
			level = 'intermediate';
		break;
		case 'avancado':
			level = 'advanced';
		break;
		case 'expert':
			level = 'expert';
		break;
		default:
			$state.go('aprenda.home');
		break;
	}

	$scope.level = $stateParams.level;

	game.loadVideo().then(
		function(videoId){
			$scope.aSource =  $sce.trustAsResourceUrl('http://api.vagalume.com.br/playerYoutubeAngular.html?id='+videoId+'&auto=0&w=100%&h=100%');
		},
		function(error){
			finish(false);
			$scope.ready = false;
			$scope.gameError = true;
			$scope.loading.message = 'Não foi possível reproduzir este jogo';
			$scope.$apply();
		}
	);

	// Executa apenas quando o player do Youtube está carregado por completo
	var sID = $rootScope.sID || false;
	eventMap.vagalumeReady = $rootScope.$on('vagalume.player.ready', function(){
		game
			.loadSubtitles(level, sID)
			.then(function(){
				finish(true);
				$scope.settings.score = 0;
				$scope.settings.goal = game.progressTotal;
			}, function(e) {
				finish(false);
			});
	});
	eventMap.vagalumeReady = $rootScope.$on('vagalume.player.error', function(){
		$scope.errorVideo = true;
		$scope.startGame = false;
		$scope.loading.message = 'Ocorreu um erro, procurando outro vídeo...'
		$timeout(function() {			
			var oldVideo = $scope.game.videoId
			,   index    = $scope.game.videoList.indexOf(oldVideo);
			if (index >= 0) {
				$scope.game.videoList.splice(index, 1);
			}
			if ($scope.game.videoList.length) {
				$scope.aSource = $sce.trustAsResourceUrl('http://api.vagalume.com.br/playerYoutubeAngular.html?id='+game.videoList[0]+'&auto=0&w=100%&h=100%');
				game.videoId = game.videoList[0];
				
				$scope.errorVideo = false;
				$scope.loading.message = 'Toque no botão para iniciar o jogo!'
			}
		}, 1000);
	});
	eventMap.vagalumePlaying = $rootScope.$on('vagalume.player.playing', function(){
		if (!$scope.loading.gameStarted) {
			$scope.loading.gameStarted = true;
			$ionicAnalytics.track('Jogo', {
				'modo': 'vídeo',
				'status': 'iniciou',
				'música': $stateParams.artUrl+'/'+$stateParams.musUrl,
				'nível':$stateParams.level
			});
		}
	});

	$scope.verifyWord = function(text, skip){
		if(window.Connection) {
			if(navigator.connection.type != Connection.NONE) {
				var res = game.setAnswer(text, skip);
				if(!res) {
					$cordovaVibration.vibrate(200);
					$scope.shakeWord = text;
					if(!text) {
						$scope.firstB.removeClass('current-blank');
						$scope.firstS.removeClass('current-shade');
						$timeout(function() {
							$scope.firstB = angular.element(document.querySelector('.subtitle-control span b'));
							if($scope.firstB) $scope.firstB.addClass('current-blank');

							$scope.firstS = angular.element(document.querySelector('.subtitle-control .bg-fade b'));
							if ($scope.firstS) $scope.firstS.addClass('current-shade');
						}, 10);
					}
					$timeout(function() {
						$scope.shakeWord = '';
					}, 500);
				} else {
					if($scope.game.answerWait) {
						$scope.firstB.removeClass('current-blank');
						$scope.firstS.removeClass('current-shade');
						$timeout(function() {
							$scope.firstB = angular.element(document.querySelector('.subtitle-control span b'));
							if($scope.firstB) $scope.firstB.addClass('current-blank');

							$scope.firstS = angular.element(document.querySelector('.subtitle-control .bg-fade b'));
							if ($scope.firstS) $scope.firstS.addClass('current-shade');
						}, 10);						
					}
				}
			} else {
				$scope.menuPopup('video');
				$cordovaToast.showShortBottom('Verifique a conexão com a internet').then(function(success) {
				}, function (error) {
				});
			}
		}
	}

	$scope.repeat = function() {
		game.repeat();
	}

	// Modal do final de jogo do modo vídeo
	$ionicModal.fromTemplateUrl('app/Game/Video/gameover-modal.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function(modal) {
		$scope.modal = modal;
	});

	// Popup de login
	$scope.loginPopup = function() {
		var myPopup = $ionicPopup.show({
			template: 'Você precisa logar no Vagalume para poder salvar seu jogo no Vagalume Aprenda.',
			cssClass: 'login-popup',
			title: 'Atenção',
			scope: $scope,
			buttons: [
				{
					text: 'Login',
					onTap: function(e) {
						e.preventDefault();
						myPopup.close();
						$state.go('login')
					}
				},
				{
					text: 'Continuar',
					type: 'button-positive',
					onTap: function(e) {
						myPopup.close();
					}
				}
			]
		});
	};

	function openGameOverModal() {
		$ionicAnalytics.track('Jogo', {
			'modo': 'vídeo',
			'status': 'finalizou',
			'música': $stateParams.artUrl+'/'+$stateParams.musUrl,
			'nível':$stateParams.level
		});
		
		$rootScope.gameOver = true;
		$scope.gameRank = null;
		$scope.moreGames = null;
		$scope.aScore = 0;
		
		if($scope.user.login) {
			GameModel.terminate($scope.game.score.score, $scope.game.songId, 'video', function(data){
				if(data.score) {
					$scope.totalScore = data.score - $scope.game.score.score;
				} else {
					$scope.totalScore = 0;
				}
			});			
		} else {
			GameModel.terminate(null, $scope.game.songId, 'video', function(){});
		}

		GameModel.getGameRank($scope.game.songId, 'video', function(rank) {
			$scope.gameRank = rank;
		});

		RankModel.getMostPlayed(function(data) {
			$scope.moreGames = data;
		});

		if(game.score.score <= 0) {
			$scope.modalTitle = 'Tente novamente!'
		} else {
			$scope.modalTitle = 'Parabéns!'
		}

		var endAnimateScore = function() {
			var score      = parseInt($scope.totalScore)
			,   totalScore = parseInt($scope.gameScore)
			,   gameScore  = parseInt($scope.game.score.score);
			$timeout(function() {
				if(score < totalScore) {
					$scope.totalScore++;
					$scope.aScore--;
					endAnimateScore();
				}
			}, $scope.game.score.score > 100 ? 5 : 20);
		},  animateScore = function() {
			if($scope.settings.score) {
				$timeout(function() {
					if($scope.aScore < $scope.settings.score) {
						$scope.aScore++;
						animateScore();
					} else {
						$timeout(function() {
							$scope.gameScore = parseInt($scope.totalScore) + parseInt($scope.settings.score);
							if($scope.user.login) {
								endAnimateScore();							
							} else if($scope.game.score.score > 0) {
								$scope.loginPopup();
							}
						}, 2000);
					}
				}, $scope.game.score.score > 100 ? 5 : 20);
			}
		}

		animateScore();
		$scope.modal.show();
	};

	function closeGameOverModal() {
		$scope.modal.remove();
	};

	// Eventos do modal de final de jogo
	$scope.$on('$destroy', function() {
		$scope.loginPopup = null;
		$scope.modal.remove();
	});
	$scope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams){
		$scope.loginPopup = null;
		if(toState != fromState) {
			if(toParams != fromParams) {
				$rootScope.sID = null;
				$rootScope.videoId = null;
			}
		}
		$scope.modal.remove();
	});

	$scope.$on('$destroy', function() {
		for (var name in eventMap) {
			eventMap[name]();
		}
	});
}])

.controller('GameVerseController', [
	'$scope', '$state', '$timeout', 'GamePhrase', 'GameModel', '$stateParams', '$ionicScrollDelegate', '$ionicModal', '$ionicPopup', 'RankModel', '$rootScope', '$cordovaVibration', '$ionicAnalytics',
	function($scope, $state, $timeout, GamePhrase, GameModel, $stateParams, $ionicScrollDelegate, $ionicModal, $ionicPopup, RankModel, $rootScope, $cordovaVibration, $ionicAnalytics) {
	$scope.loading = {
		finished: false,
		message:  ''
	};

	var inTransition = null
	,   clicked      = null
	,   eventMap     = {};

	$scope.aShadow = {
		before: false,
		after: true
	};

	$rootScope.gameOver = false;

	// Verifica a conexão com a internet
	if (window.Connection) {
		if(navigator.connection.type == Connection.NONE) {
			$state.go('aprenda.no-connection');
		}
	}

	// Verificação de level da URL
	switch($stateParams.level) {
		case 'iniciante':
			level = 'beginner';
		break;
		case 'intermediario':
			level = 'intermediate';
		break;
		case 'avancado':
			level = 'advanced';
		break;
		case 'expert':
			level = 'expert';
		break;
		default:
			$state.go('aprenda.home');
		break;
	}

	$scope.level = $stateParams.level;

	$scope.hideMenuBar = false;
	$scope.hideMenu = function() {
		$scope.popover.hide();
		$scope.hideMenuBar = true;
	}

	$scope.callFullScreen = function(){
		$scope.popover.hide();
		if ( document.fullscreenEnabled || document.webkitFullscreenEnabled
		||   document.mozFullScreenEnabled || document.msFullscreenEnabled
		) {
			if ( document.fullscreenElement || document.webkitFullscreenElement
			||   document.mozFullScreenElement || document.msFullscreenElement
			) {
				$scope.isFullScreen = false;
				var i = document;
				var callfs = i.exitFullscreen || i.webkitExitFullscreen || i.mozCancelFullScreen || i.msExitFullscreen;
			} else{
				$scope.isFullScreen = true;
				var i = document.querySelector('.pane');
				var callfs = i.requestFullscreen || i.webkitRequestFullscreen || i.mozRequestFullScreen || i.msRequestFullscreen;
			}
			if (callfs) callfs.apply(i);
		}
	}

	$scope.startGame = false;
	$scope.wrongAnswer = false;
	$scope.ready = true;

	eventMap.phraseOptions = $rootScope.$on('phraseOptions.move', function() {
		inTransition = $timeout.cancel(inTransition);
		inTransition =  $timeout(function() {
			$timeout.cancel(inTransition);
			inTransition = null;
			if(clicked){
				$scope.getNextPhrase(clicked);
			}
			clicked = null;
		}, 1500);
	});

	$scope.$on('$destroy', function() {
		for (var name in eventMap) {
			eventMap[name]();
		}
	});

	$scope.getNextPhrase = function(type) {
		if(!inTransition) {
			$scope.game.getNextPhrase(type);
		} else {
			clicked = type;
		}
	}

	var loading = function() {

	}
	,   finish = function(data) {
		if(data) {
			$timeout(function() {
				$scope.ready = true;
				$scope.loading.finished = true;
				$scope.loading.message = 'Toque no botão para iniciar o jogo!';
				console.log($scope.game.songID)
				$ionicAnalytics.track('Jogo', {
					'modo': 'texto',
					'status': 'iniciou',
					'música': $stateParams.artUrl+'/'+$stateParams.musUrl,
					'nível': $stateParams.level
				});
			}, 1000);
		} else {
			$timeout(function() {
				$scope.ready = false;
				$scope.gameError = true;
				$scope.loading.message = 'Música sem tradução.';				
			}, 500);
		}
	}
	,   removeDoneList = function() {
		var li = document.querySelectorAll('.users-current li');
		if(li.length) {
			var ul = li[0].parentNode;
			for (var i = 0; i < li.length; i++) {
				ul.removeChild(li[i]);
			};			
		}
	}

	$scope.game = {};
	GamePhrase
		.run($stateParams.artUrl, $stateParams.musUrl, $stateParams.level)
		.then(
			function(game){
				finish(true);
				$scope.game = game;
				$timeout(function() {
					$scope.scrollTo();
				}, 100);
				game.on = function(type){
					switch(type) {
						case 'end':
							$timeout(function() {
								$rootScope.gameOver = true;								
								$scope.openGameOverModal();
							}, 1000);
							break;
						case 'changeVerse':
							$timeout(function() {
								$scope.scrollTo();
							}, 100);
							break;
						case 'wrongAnswer':
							$cordovaVibration.vibrate(200);
							$scope.wrongAnswer = true;
							$timeout(function() {
								$scope.wrongAnswer = false;
							}, 500);			
							break;
					}
				}
			},
			function(error){
				finish(false);
			}
		);

	$scope.scrollTo = function() {
		var pos = document.querySelector('.current-verse .verse.current').getBoundingClientRect().top - 60;
		if (pos < 150) {
			var contentTop = document.querySelector('.land-top > div:nth-child(1)').getBoundingClientRect().top -3
			,   top = $ionicScrollDelegate.getScrollPosition().top;
			while(pos < contentTop) {
				top--;
				pos++;
			}
			$ionicScrollDelegate.scrollTo(0, top, true);
		} else if (pos > 160) {
			var contentTop = document.querySelector('.land-top > div:nth-child(1)').getBoundingClientRect().top -3
			,   top = $ionicScrollDelegate.getScrollPosition().top;
			while(pos > contentTop) {
				top++
				pos--;
			}
			$ionicScrollDelegate.scrollTo(0, top, true);
		}
	};

	$scope.onScroll = function() {
		var scrollTop    = $ionicScrollDelegate.getScrollPosition().top
		,   scrollHeight = document.querySelector('.current-verse .scroll').offsetHeight;

		if (scrollTop == 0) {
			$scope.aShadow.before = false;
			$scope.$apply();
		} else if (scrollTop != 0) {
			$scope.aShadow.before = true;
			$scope.aShadow.after = true;
			$scope.$apply();
		}
	}

	// Popup de login
	$scope.loginPopup = function() {
		var myPopup = $ionicPopup.show({
			template: 'Você precisa logar no Vagalume para poder salvar seu jogo no Vagalume Aprenda.',
			cssClass: 'login-popup',
			title: 'Atenção',
			scope: $scope,
			buttons: [
				{
					text: 'Login',
					onTap: function(e) {
						e.preventDefault();
						myPopup.close();
						$state.go('login')
					}
				},
				{
					text: 'Continuar',
					type: 'button-positive',
					onTap: function(e) {
						myPopup.close();
					}
				}
			]
		});
	};

	// Modal do final de jogo do modo frases
	$ionicModal.fromTemplateUrl('app/Game/Verse/gameover-modal.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function(modal) {
		$scope.modal = modal;
	});

	$scope.openGameOverModal = function() {
		$ionicAnalytics.track('Jogo', {
			'modo': 'texto',
			'status': 'finalizou',
			'música': $stateParams.artUrl+'/'+$stateParams.musUrl,
			'nível': $stateParams.level
		});
		$scope.gameRank = null;
		$scope.aScore   = 0;
		if($scope.user.login) {
			GameModel.terminate($scope.game.score.score, $scope.game.songID, 'verse', function(data) {
				if(data.score) {
					$scope.totalScore = data.score - $scope.game.score.score;
				} else {
					$scope.totalScore = 0;
				}
			});		
		} else {
			GameModel.terminate(null, this.songID, 'verse', function(res) {});
		}

		GameModel.getGameRank($scope.game.songID, 'verse', function(rank) {
			$scope.gameRank = rank;
		});

		RankModel.getMostPlayed(function(data) {
			$scope.moreGames = data;
		});

		if($scope.game.score.score <= 0) {
			$scope.modalTitle = 'Tente novamente!'
		} else {
			$scope.modalTitle = 'Parabéns!'
		}

		var endAnimateScore = function() {
			var score      = parseInt($scope.totalScore)
			,   totalScore = parseInt($scope.gameScore)
			,   gameScore  = parseInt($scope.game.score.score);
			$timeout(function() {
				if(score < totalScore) {
					$scope.totalScore++;
					$scope.aScore--;
					endAnimateScore();
				}
			}, $scope.game.score.score > 100 ? 5 : 20);
		},  animateScore = function() {
			if($scope.game.score.score) {
				$timeout(function() {
					if($scope.aScore < $scope.game.score.score) {
						$scope.aScore++;
						animateScore();
					} else {
						$timeout(function() {
							$scope.gameScore = parseInt($scope.totalScore) + parseInt($scope.game.score.score);
							if($scope.user.login) {
								endAnimateScore();							
							} else if($scope.game.score.score > 0) {
								$scope.loginPopup();
							}
						}, 2000);
					}
				}, $scope.game.score.score > 100 ? 5 : 20);
			}
		}

		animateScore();
		$scope.modal.show();
	};

	$scope.closeGameOverModal = function() {
		$scope.modal.remove();
	};

	// Eventos do modal de final de jogo
	$scope.$on('$destroy', function() {
		$scope.loginPopup = null;
		$scope.modal.remove();
	});	
	$scope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams){
		// Removendo modal de fim de jogo
		$scope.modal.remove();
	});
}])

.controller('SuggestController', ['$scope', '$state', 'RequestModel', function($scope, $state, RequestModel){
	$scope.suggestData  = { "songs" : [], "search" : '' };

	// Suggest de pesquisa de músicas do menu
	$scope.suggest = function() {
		RequestModel
		.doSuggest($scope.suggestData.search, function(data){
			for (var i = 0; i < data.response.docs.length; i++) {
				data.response.docs[i].artUrl = data.response.docs[i].url.match(/^\/([^\/]+)\/([^\/.]+)/)[1];
				data.response.docs[i].musUrl = data.response.docs[i].url.match(/^\/([^\/]+)\/([^\/.]+)/)[2];
			};
			$scope.suggestData.songs = data.response.docs;
		})
	}
}])

.controller('RankController', [
	'$scope', '$state', 'RankModel', '$ionicLoading', '$ionicSlideBoxDelegate', '$ionicScrollDelegate',
	function($scope, $state, RankModel, $ionicLoading, $ionicSlideBoxDelegate, $ionicScrollDelegate) {
	var resultWeekRank = function(data) {
		var weeks = Object.keys(data);
		var ident = []
		for (var i = 0; i < weeks.length; i++) {
			weeks[i] = {key: weeks[i].slice(-2), value: 'Semana ' + weeks[i].slice(-2)};
		};
		$scope.weeksRange = weeks;
		$scope.allWeekRank = data;
		$scope.weekRank = $scope.allWeekRank[$scope.selectedYear.toString() + $scope.selectedWeek.toString()];
		finish();
	}
	,   getStartEndWeekDay = function(date) {
		var curr = date;
		var first = curr.getDate() - curr.getDay();
		var last = first + 6;

		var firstday = new Date(curr.setDate(first)).toUTCString();
		var lastday = new Date(curr.setDate(last)).toUTCString();

		return {
			start: firstday,
			end: lastday
		}
	}
	,   resultMonthRank = function(data) {
		$scope.allMonthRank = data;
		$scope.monthRank = $scope.allMonthRank[$scope.selectedYear + $scope.selectedMonth];
		finish();
	}
	,   resultGeneralRank = function(data) {
		$scope.generalRank = data;
		finish();
	}
	,   getRank = function(action, year, force) {
		$scope.activeTab = action;
		switch(action) {
			case 'week':
				if (!$scope.weekRank) loading();
				if (!$scope.weekRank || force) RankModel.getYearWeekRank(year, resultWeekRank);
				$ionicSlideBoxDelegate.slide(0);
				break;
			case 'month':
				if (!$scope.monthRank) loading();
				if (!$scope.monthRank || force) RankModel.getYearMonthRank(year, resultMonthRank);
				$ionicSlideBoxDelegate.slide(1);
				break;
			case 'general':
				if (!$scope.generalRank) loading();
				if (!$scope.generalRank || force) RankModel.getGeneralRank(resultGeneralRank);
				$ionicSlideBoxDelegate.slide(2);
				break;
		}
	}
	,   changeMonth = function() {
		$scope.monthRank = $scope.allMonthRank[$scope.selectedYear + $scope.selectedMonth];
	}
	,   changeYear = function() {
		switch($scope.activeTab) {
			case 'week':
				getRank('week', $scope.selectedYear, true);
				break;
			case 'month':
				getRank('month', $scope.selectedYear, true);
				break;
		}
	}
	,   changeWeek = function() {
		$scope.weekRank = $scope.allWeekRank[$scope.selectedYear + $scope.selectedWeek];
	}
	,   loading = function() {
		$ionicLoading.show({template: '<ion-spinner icon="ios" class="spinner-calm"></ion-spinner>', noBackdrop: true});
	}
	,   finish = function() {
		$ionicLoading.hide();
	}

	$scope.clickTab = getRank;
	$scope.changeMonth = changeMonth;
	$scope.changeYear = changeYear;
	$scope.changeWeek = changeWeek;

	$scope.weekRank = $scope.monthRank = $scope.generalRank = false;
	$scope.activeTab = '';

	var date  = new Date(),
		year  = date.getFullYear(),
		month = ((date.getMonth() + 1) < 10 ? '0' : '') + (date.getMonth() + 1)

	$scope.selectedYear  = year.toString();
	$scope.selectedMonth = month.toString();
	$scope.selectedWeek  = date.getWeek().toString();
	$scope.slideIndex = 0;

	if (window.Connection) {
		if(navigator.connection.type == Connection.NONE) {
			$state.go('aprenda.no-connection');
		}
	}

	$scope.slideChanged = function(index, force) {
		$ionicScrollDelegate.scrollTop();
		$scope.slideIndex = index;
		switch(index) {
			case 0:
				if (!$scope.weekRank) loading();
				if (!$scope.weekRank || force) RankModel.getYearWeekRank(year, resultWeekRank);
				$scope.activeTab = 	'week';
				break;
			case 1:
				if (!$scope.monthRank) loading();
				if (!$scope.monthRank || force) RankModel.getYearMonthRank(year, resultMonthRank);
				$scope.activeTab = 'month';
				break;
			case 2:
				if (!$scope.generalRank) loading();
				if (!$scope.generalRank || force) RankModel.getGeneralRank(resultGeneralRank);
				$scope.activeTab = 'general';
				break;
		}
	};

	getRank('week', $scope.selectedYear);
}])

.controller('RulesController', [
	'$scope', '$ionicLoading', '$ionicSlideBoxDelegate', '$ionicScrollDelegate',
	function($scope, $ionicLoading, $ionicSlideBoxDelegate, $ionicScrollDelegate) {
	var changeSection = function(action) {
		$scope.activeTab = action;
		switch(action) {
			case 'about':
				$ionicSlideBoxDelegate.slide(0);
				break;
			case 'score':
				$ionicSlideBoxDelegate.slide(2);
				break;
			case 'rules':
				$ionicSlideBoxDelegate.slide(1);
				break;
			case 'faq':
				$ionicSlideBoxDelegate.slide(3);
				break;
		}
	}

	$scope.slideIndex = 0;

	$scope.slideChanged = function(index) {
		$ionicScrollDelegate.scrollTop();
		$scope.slideIndex = index;
	};

	$scope.clickTab = changeSection;
	$scope.activeTab = 'about';
}])

.controller('LoginController',function($scope, sessionUser, $state, $ionicPopup, $cordovaToast, $ionicAnalytics){
	function loginSuccess(data, status){
		if(data){
			if (data.code == 'LOGGED') {
				$ionicAnalytics.track('Efetuar login');
				$state.go('aprenda.home');				
			} else {
				$cordovaToast.showShortBottom(data.message);
			}
		} else {
			if(data.message){
				$ionicPopup.alert({'title':'Alerta','template':data.message});
			}
		}
	}
	$scope.funcSubmit = function(){
		$scope.loginMessage = '';
		// Verifica a conexão com a internet
		if (window.Connection) {
			if(navigator.connection.type == Connection.NONE) {
				$cordovaToast.showShortCenter('Verifique a conexão com a internet').then(function(success) {
				}, function (error) {
				});
			} else {
				sessionUser.login($scope.dataUser, $scope.dataPass, loginSuccess);				
			}
		}
	}
	$scope.connectFacebook = function(){
		$scope.loginMessage = '';
		// Verifica a conexão com a internet
		if (window.Connection) {
			if(navigator.connection.type == Connection.NONE) {
				$cordovaToast.showShortBottom('Verifique a conexão com a internet').then(function(success) {
				}, function (error) {
				});
			} else {
				sessionUser.connect('facebook', loginSuccess);				
			}
		}
	}
	$scope.connectGoogle = function(){
		$scope.loginMessage = '';
		// Verifica a conexão com a internet
		if (window.Connection) {
			if(navigator.connection.type == Connection.NONE) {
				$cordovaToast.showShortBottom('Verifique a conexão com a internet').then(function(success) {
				}, function (error) {
				});
			} else {
				sessionUser.connect('google', loginSuccess);				
			}
		}
	}
})

.controller('LangController', ['$scope', '$state', '$stateParams', 'RequestModel', function($scope, $state, $stateParams, RequestModel) {
	var lang   = $stateParams.lang
	,   langID = 0 ;
	$scope.template = {};
	$scope.suggest = {
		search: ''
	};
	$scope.featuredSongs = [];
	switch(lang) {
		case 'ingles':
			$scope.template.title    = 'Aprenda Inglês';
			$scope.template.langName = 'inglês';
			langID = 2;
			break;
		case 'espanhol':
			$scope.template.title    = 'Aprenda Espanhol';
			$scope.template.langName = 'espanhol';
			langID = 3;
			break;
		case 'frances':
			$scope.template.title    = 'Aprenda Francês';
			$scope.template.langName = 'francês';
			langID = 4;
			break;
		case 'portugues':
			$scope.template.title    = 'Aprenda Português';
			$scope.template.langName = 'português';
			langID = 1;
			break;
		case 'alemao':
			$scope.template.title    = 'Aprenda Alemão';
			$scope.template.langName = 'alemão';
			langID = 5;
			break;
		default:
			$state.go('aprenda.home');
	}
	$scope.langURL = lang;
	$scope.template.url = 'lang.html';

	$scope.suggestLang = function() {
		RequestModel
		.doSuggest($scope.suggest.search, function(data){
			if (data.response.docs.length) {
				$scope.searchHome = true;
				var docs = data.response.docs
				,   matchRE = /^\/([^\/]+)\/([^\/.]+)/
				,   match;
				for (var i = 0; i < docs.length; i++) {
					match = docs[i].url.match(matchRE);
					docs[i].artUrl = match[1];
					docs[i].musUrl = match[2];
				};
				match = matchRE = undefined;
				$scope.suggest.songs = docs;
			} else {
				$scope.searchHome = false;
			}
		}, langID)
	}
	
	RequestModel.getLangVideos(lang, function(songs) {
		$scope.featuredSongs = songs;
	});
}])

.controller('LostConnectionController',function($scope, $state){
	$scope.tryConnection = function() {
		if(window.Connection) {
			if(navigator.connection.type != Connection.NONE) {
				$state.go('aprenda.home');
			}
		}		
	}
})

.controller('RegisterController', function($scope, sessionUser, $state, $rootScope, $cordovaToast, $ionicAnalytics){
	if(window.Connection) {
		if(navigator.connection.type == Connection.NONE) {
			$state.go('aprenda.no-connection');
		}
	}
	$scope.dataLabelGenre = 'Masculino';
	$scope.funcBack = function(){
		$state.go('login');
	}
	$scope.funcUpdateGenre = function(){
		$scope.dataLabelGenre = $scope.dataGenre ? 'Feminino' : 'Masculino';
	}
	$scope.funcRegister = function(){
		if(window.Connection) {
			if(navigator.connection.type == Connection.NONE) {
				$cordovaToast.showShortCenter('Verifique a conexão com a internet').then(function(success) {
				}, function (error) {
				});
			}
		}
		//'name':'gender':'email':'emailconf':'pass':'passconf':'agree':'nameshow':'login':'aboutMe':
		$scope.registerFullNameAlert = $scope.registerGenreAlert = $scope.registerEmailAlert = '';
		$scope.registerPassAlert = $scope.registerTermsAlert = $scope.registerShowNameAlert = '';
		$scope.registerLoginAlert = $scope.registerSubmitAlert = '';
		sessionUser.register(
			{
				name      :$scope.dataFullName,
				gender    :$scope.dataGenre ? 'female' : 'male',
				email     :$scope.dataEmail,
				emailconf :$scope.dataEmail,
				pass      :$scope.dataPass,
				passconf  :$scope.dataPass,
				agree     :$scope.dataTerms,
				nameshow  :$scope.dataShowName,
				login     :$scope.dataLogin,
				aboutMe   :''
			},
			function(success, error){
				if(success){
					$ionicAnalytics.track('Efetuar cadastro');
					$state.go('aprenda.home');
				} else {
					switch(error.field){
						case 'name':
							$scope.registerFullNameAlert = error.message;
							break;
						case 'gender':
							$scope.registerGenreAlert = error.message;
							break;
						case 'email':
							$scope.registerEmailAlert = error.message;
							break;
						case 'pass':
						case 'passconf':
							$scope.registerPassAlert = error.message;
							break;
						case 'agree':
							$scope.registerTermsAlert = error.message;
							break;
						case 'name_show':
						case 'nameshow':
							$scope.registerShowNameAlert = error.message;
							break;
						case 'login':
							$scope.registerLoginAlert = error.message;
							break;
						case 'timeout':
							break;
					}
					if(error.field){
						$scope.registerSubmitAlert = 'Alguma informação está incorreta, favor corrigir e tentar novamente';
					}
				}
				$scope.$digest(); // Atualiza a view sem a necessidade de interaè¤¯ do usuâ³©o
			}
		);
	}
});
